from enum import Enum
import math
import string


class ReactivePowerUnits(Enum):
        """
            ReactivePowerUnits enumeration
        """
        
        VoltampereReactive = 'voltampere_reactive'
        """
            
        """
        
        KilovoltampereReactive = 'kilovoltampere_reactive'
        """
            
        """
        
        MegavoltampereReactive = 'megavoltampere_reactive'
        """
            
        """
        
        GigavoltampereReactive = 'gigavoltampere_reactive'
        """
            
        """
        

class ReactivePower:
    """
    Volt-ampere reactive (var) is a unit by which reactive power is expressed in an AC electric power system. Reactive power exists in an AC circuit when the current and voltage are not in phase.

    Args:
        value (float): The value.
        from_unit (ReactivePowerUnits): The ReactivePower unit to create from, The default unit is VoltampereReactive
    """
    def __init__(self, value: float, from_unit: ReactivePowerUnits = ReactivePowerUnits.VoltampereReactive):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__voltamperes_reactive = None
        
        self.__kilovoltamperes_reactive = None
        
        self.__megavoltamperes_reactive = None
        
        self.__gigavoltamperes_reactive = None
        

    def __convert_from_base(self, from_unit: ReactivePowerUnits) -> float:
        value = self.__value
        
        if from_unit == ReactivePowerUnits.VoltampereReactive:
            return (value)
        
        if from_unit == ReactivePowerUnits.KilovoltampereReactive:
            return ((value) / 1000.0)
        
        if from_unit == ReactivePowerUnits.MegavoltampereReactive:
            return ((value) / 1000000.0)
        
        if from_unit == ReactivePowerUnits.GigavoltampereReactive:
            return ((value) / 1000000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ReactivePowerUnits) -> float:
        
        if to_unit == ReactivePowerUnits.VoltampereReactive:
            return (value)
        
        if to_unit == ReactivePowerUnits.KilovoltampereReactive:
            return ((value) * 1000.0)
        
        if to_unit == ReactivePowerUnits.MegavoltampereReactive:
            return ((value) * 1000000.0)
        
        if to_unit == ReactivePowerUnits.GigavoltampereReactive:
            return ((value) * 1000000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_voltamperes_reactive(voltamperes_reactive: float):
        """
        Create a new instance of ReactivePower from a value in voltamperes_reactive.

        

        :param meters: The ReactivePower value in voltamperes_reactive.
        :type voltamperes_reactive: float
        :return: A new instance of ReactivePower.
        :rtype: ReactivePower
        """
        return ReactivePower(voltamperes_reactive, ReactivePowerUnits.VoltampereReactive)

    
    @staticmethod
    def from_kilovoltamperes_reactive(kilovoltamperes_reactive: float):
        """
        Create a new instance of ReactivePower from a value in kilovoltamperes_reactive.

        

        :param meters: The ReactivePower value in kilovoltamperes_reactive.
        :type kilovoltamperes_reactive: float
        :return: A new instance of ReactivePower.
        :rtype: ReactivePower
        """
        return ReactivePower(kilovoltamperes_reactive, ReactivePowerUnits.KilovoltampereReactive)

    
    @staticmethod
    def from_megavoltamperes_reactive(megavoltamperes_reactive: float):
        """
        Create a new instance of ReactivePower from a value in megavoltamperes_reactive.

        

        :param meters: The ReactivePower value in megavoltamperes_reactive.
        :type megavoltamperes_reactive: float
        :return: A new instance of ReactivePower.
        :rtype: ReactivePower
        """
        return ReactivePower(megavoltamperes_reactive, ReactivePowerUnits.MegavoltampereReactive)

    
    @staticmethod
    def from_gigavoltamperes_reactive(gigavoltamperes_reactive: float):
        """
        Create a new instance of ReactivePower from a value in gigavoltamperes_reactive.

        

        :param meters: The ReactivePower value in gigavoltamperes_reactive.
        :type gigavoltamperes_reactive: float
        :return: A new instance of ReactivePower.
        :rtype: ReactivePower
        """
        return ReactivePower(gigavoltamperes_reactive, ReactivePowerUnits.GigavoltampereReactive)

    
    @property
    def voltamperes_reactive(self) -> float:
        """
        
        """
        if self.__voltamperes_reactive != None:
            return self.__voltamperes_reactive
        self.__voltamperes_reactive = self.__convert_from_base(ReactivePowerUnits.VoltampereReactive)
        return self.__voltamperes_reactive

    
    @property
    def kilovoltamperes_reactive(self) -> float:
        """
        
        """
        if self.__kilovoltamperes_reactive != None:
            return self.__kilovoltamperes_reactive
        self.__kilovoltamperes_reactive = self.__convert_from_base(ReactivePowerUnits.KilovoltampereReactive)
        return self.__kilovoltamperes_reactive

    
    @property
    def megavoltamperes_reactive(self) -> float:
        """
        
        """
        if self.__megavoltamperes_reactive != None:
            return self.__megavoltamperes_reactive
        self.__megavoltamperes_reactive = self.__convert_from_base(ReactivePowerUnits.MegavoltampereReactive)
        return self.__megavoltamperes_reactive

    
    @property
    def gigavoltamperes_reactive(self) -> float:
        """
        
        """
        if self.__gigavoltamperes_reactive != None:
            return self.__gigavoltamperes_reactive
        self.__gigavoltamperes_reactive = self.__convert_from_base(ReactivePowerUnits.GigavoltampereReactive)
        return self.__gigavoltamperes_reactive

    
    def to_string(self, unit: ReactivePowerUnits = ReactivePowerUnits.VoltampereReactive) -> string:
        """
        Format the ReactivePower to string.
        Note! the default format for ReactivePower is VoltampereReactive.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ReactivePowerUnits.VoltampereReactive:
            return f"""{self.voltamperes_reactive} var"""
        
        if unit == ReactivePowerUnits.KilovoltampereReactive:
            return f"""{self.kilovoltamperes_reactive} """
        
        if unit == ReactivePowerUnits.MegavoltampereReactive:
            return f"""{self.megavoltamperes_reactive} """
        
        if unit == ReactivePowerUnits.GigavoltampereReactive:
            return f"""{self.gigavoltamperes_reactive} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: ReactivePowerUnits = ReactivePowerUnits.VoltampereReactive) -> string:
        """
        Get ReactivePower unit abbreviation.
        Note! the default abbreviation for ReactivePower is VoltampereReactive.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ReactivePowerUnits.VoltampereReactive:
            return """var"""
        
        if unit_abbreviation == ReactivePowerUnits.KilovoltampereReactive:
            return """"""
        
        if unit_abbreviation == ReactivePowerUnits.MegavoltampereReactive:
            return """"""
        
        if unit_abbreviation == ReactivePowerUnits.GigavoltampereReactive:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, ReactivePower):
            raise TypeError("unsupported operand type(s) for +: 'ReactivePower' and '{}'".format(type(other).__name__))
        return ReactivePower(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, ReactivePower):
            raise TypeError("unsupported operand type(s) for *: 'ReactivePower' and '{}'".format(type(other).__name__))
        return ReactivePower(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, ReactivePower):
            raise TypeError("unsupported operand type(s) for -: 'ReactivePower' and '{}'".format(type(other).__name__))
        return ReactivePower(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, ReactivePower):
            raise TypeError("unsupported operand type(s) for /: 'ReactivePower' and '{}'".format(type(other).__name__))
        return ReactivePower(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, ReactivePower):
            raise TypeError("unsupported operand type(s) for %: 'ReactivePower' and '{}'".format(type(other).__name__))
        return ReactivePower(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, ReactivePower):
            raise TypeError("unsupported operand type(s) for **: 'ReactivePower' and '{}'".format(type(other).__name__))
        return ReactivePower(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, ReactivePower):
            raise TypeError("unsupported operand type(s) for ==: 'ReactivePower' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, ReactivePower):
            raise TypeError("unsupported operand type(s) for <: 'ReactivePower' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, ReactivePower):
            raise TypeError("unsupported operand type(s) for >: 'ReactivePower' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, ReactivePower):
            raise TypeError("unsupported operand type(s) for <=: 'ReactivePower' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, ReactivePower):
            raise TypeError("unsupported operand type(s) for >=: 'ReactivePower' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class RotationalStiffnessPerLengthUnits(Enum):
        """
            RotationalStiffnessPerLengthUnits enumeration
        """
        
        NewtonMeterPerRadianPerMeter = 'newton_meter_per_radian_per_meter'
        """
            
        """
        
        PoundForceFootPerDegreesPerFoot = 'pound_force_foot_per_degrees_per_foot'
        """
            
        """
        
        KilopoundForceFootPerDegreesPerFoot = 'kilopound_force_foot_per_degrees_per_foot'
        """
            
        """
        
        KilonewtonMeterPerRadianPerMeter = 'kilonewton_meter_per_radian_per_meter'
        """
            
        """
        
        MeganewtonMeterPerRadianPerMeter = 'meganewton_meter_per_radian_per_meter'
        """
            
        """
        

class RotationalStiffnessPerLength:
    """
    https://en.wikipedia.org/wiki/Stiffness#Rotational_stiffness

    Args:
        value (float): The value.
        from_unit (RotationalStiffnessPerLengthUnits): The RotationalStiffnessPerLength unit to create from, The default unit is NewtonMeterPerRadianPerMeter
    """
    def __init__(self, value: float, from_unit: RotationalStiffnessPerLengthUnits = RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__newton_meters_per_radian_per_meter = None
        
        self.__pound_force_feet_per_degrees_per_feet = None
        
        self.__kilopound_force_feet_per_degrees_per_feet = None
        
        self.__kilonewton_meters_per_radian_per_meter = None
        
        self.__meganewton_meters_per_radian_per_meter = None
        

    def __convert_from_base(self, from_unit: RotationalStiffnessPerLengthUnits) -> float:
        value = self.__value
        
        if from_unit == RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter:
            return (value)
        
        if from_unit == RotationalStiffnessPerLengthUnits.PoundForceFootPerDegreesPerFoot:
            return (value / 254.864324570)
        
        if from_unit == RotationalStiffnessPerLengthUnits.KilopoundForceFootPerDegreesPerFoot:
            return (value / 254864.324570)
        
        if from_unit == RotationalStiffnessPerLengthUnits.KilonewtonMeterPerRadianPerMeter:
            return ((value) / 1000.0)
        
        if from_unit == RotationalStiffnessPerLengthUnits.MeganewtonMeterPerRadianPerMeter:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RotationalStiffnessPerLengthUnits) -> float:
        
        if to_unit == RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter:
            return (value)
        
        if to_unit == RotationalStiffnessPerLengthUnits.PoundForceFootPerDegreesPerFoot:
            return (value * 254.864324570)
        
        if to_unit == RotationalStiffnessPerLengthUnits.KilopoundForceFootPerDegreesPerFoot:
            return (value * 254864.324570)
        
        if to_unit == RotationalStiffnessPerLengthUnits.KilonewtonMeterPerRadianPerMeter:
            return ((value) * 1000.0)
        
        if to_unit == RotationalStiffnessPerLengthUnits.MeganewtonMeterPerRadianPerMeter:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_newton_meters_per_radian_per_meter(newton_meters_per_radian_per_meter: float):
        """
        Create a new instance of RotationalStiffnessPerLength from a value in newton_meters_per_radian_per_meter.

        

        :param meters: The RotationalStiffnessPerLength value in newton_meters_per_radian_per_meter.
        :type newton_meters_per_radian_per_meter: float
        :return: A new instance of RotationalStiffnessPerLength.
        :rtype: RotationalStiffnessPerLength
        """
        return RotationalStiffnessPerLength(newton_meters_per_radian_per_meter, RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter)

    
    @staticmethod
    def from_pound_force_feet_per_degrees_per_feet(pound_force_feet_per_degrees_per_feet: float):
        """
        Create a new instance of RotationalStiffnessPerLength from a value in pound_force_feet_per_degrees_per_feet.

        

        :param meters: The RotationalStiffnessPerLength value in pound_force_feet_per_degrees_per_feet.
        :type pound_force_feet_per_degrees_per_feet: float
        :return: A new instance of RotationalStiffnessPerLength.
        :rtype: RotationalStiffnessPerLength
        """
        return RotationalStiffnessPerLength(pound_force_feet_per_degrees_per_feet, RotationalStiffnessPerLengthUnits.PoundForceFootPerDegreesPerFoot)

    
    @staticmethod
    def from_kilopound_force_feet_per_degrees_per_feet(kilopound_force_feet_per_degrees_per_feet: float):
        """
        Create a new instance of RotationalStiffnessPerLength from a value in kilopound_force_feet_per_degrees_per_feet.

        

        :param meters: The RotationalStiffnessPerLength value in kilopound_force_feet_per_degrees_per_feet.
        :type kilopound_force_feet_per_degrees_per_feet: float
        :return: A new instance of RotationalStiffnessPerLength.
        :rtype: RotationalStiffnessPerLength
        """
        return RotationalStiffnessPerLength(kilopound_force_feet_per_degrees_per_feet, RotationalStiffnessPerLengthUnits.KilopoundForceFootPerDegreesPerFoot)

    
    @staticmethod
    def from_kilonewton_meters_per_radian_per_meter(kilonewton_meters_per_radian_per_meter: float):
        """
        Create a new instance of RotationalStiffnessPerLength from a value in kilonewton_meters_per_radian_per_meter.

        

        :param meters: The RotationalStiffnessPerLength value in kilonewton_meters_per_radian_per_meter.
        :type kilonewton_meters_per_radian_per_meter: float
        :return: A new instance of RotationalStiffnessPerLength.
        :rtype: RotationalStiffnessPerLength
        """
        return RotationalStiffnessPerLength(kilonewton_meters_per_radian_per_meter, RotationalStiffnessPerLengthUnits.KilonewtonMeterPerRadianPerMeter)

    
    @staticmethod
    def from_meganewton_meters_per_radian_per_meter(meganewton_meters_per_radian_per_meter: float):
        """
        Create a new instance of RotationalStiffnessPerLength from a value in meganewton_meters_per_radian_per_meter.

        

        :param meters: The RotationalStiffnessPerLength value in meganewton_meters_per_radian_per_meter.
        :type meganewton_meters_per_radian_per_meter: float
        :return: A new instance of RotationalStiffnessPerLength.
        :rtype: RotationalStiffnessPerLength
        """
        return RotationalStiffnessPerLength(meganewton_meters_per_radian_per_meter, RotationalStiffnessPerLengthUnits.MeganewtonMeterPerRadianPerMeter)

    
    @property
    def newton_meters_per_radian_per_meter(self) -> float:
        """
        
        """
        if self.__newton_meters_per_radian_per_meter != None:
            return self.__newton_meters_per_radian_per_meter
        self.__newton_meters_per_radian_per_meter = self.__convert_from_base(RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter)
        return self.__newton_meters_per_radian_per_meter

    
    @property
    def pound_force_feet_per_degrees_per_feet(self) -> float:
        """
        
        """
        if self.__pound_force_feet_per_degrees_per_feet != None:
            return self.__pound_force_feet_per_degrees_per_feet
        self.__pound_force_feet_per_degrees_per_feet = self.__convert_from_base(RotationalStiffnessPerLengthUnits.PoundForceFootPerDegreesPerFoot)
        return self.__pound_force_feet_per_degrees_per_feet

    
    @property
    def kilopound_force_feet_per_degrees_per_feet(self) -> float:
        """
        
        """
        if self.__kilopound_force_feet_per_degrees_per_feet != None:
            return self.__kilopound_force_feet_per_degrees_per_feet
        self.__kilopound_force_feet_per_degrees_per_feet = self.__convert_from_base(RotationalStiffnessPerLengthUnits.KilopoundForceFootPerDegreesPerFoot)
        return self.__kilopound_force_feet_per_degrees_per_feet

    
    @property
    def kilonewton_meters_per_radian_per_meter(self) -> float:
        """
        
        """
        if self.__kilonewton_meters_per_radian_per_meter != None:
            return self.__kilonewton_meters_per_radian_per_meter
        self.__kilonewton_meters_per_radian_per_meter = self.__convert_from_base(RotationalStiffnessPerLengthUnits.KilonewtonMeterPerRadianPerMeter)
        return self.__kilonewton_meters_per_radian_per_meter

    
    @property
    def meganewton_meters_per_radian_per_meter(self) -> float:
        """
        
        """
        if self.__meganewton_meters_per_radian_per_meter != None:
            return self.__meganewton_meters_per_radian_per_meter
        self.__meganewton_meters_per_radian_per_meter = self.__convert_from_base(RotationalStiffnessPerLengthUnits.MeganewtonMeterPerRadianPerMeter)
        return self.__meganewton_meters_per_radian_per_meter

    
    def to_string(self, unit: RotationalStiffnessPerLengthUnits = RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter) -> string:
        """
        Format the RotationalStiffnessPerLength to string.
        Note! the default format for RotationalStiffnessPerLength is NewtonMeterPerRadianPerMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter:
            return f"""{self.newton_meters_per_radian_per_meter} N·m/rad/m"""
        
        if unit == RotationalStiffnessPerLengthUnits.PoundForceFootPerDegreesPerFoot:
            return f"""{self.pound_force_feet_per_degrees_per_feet} lbf·ft/deg/ft"""
        
        if unit == RotationalStiffnessPerLengthUnits.KilopoundForceFootPerDegreesPerFoot:
            return f"""{self.kilopound_force_feet_per_degrees_per_feet} kipf·ft/°/ft"""
        
        if unit == RotationalStiffnessPerLengthUnits.KilonewtonMeterPerRadianPerMeter:
            return f"""{self.kilonewton_meters_per_radian_per_meter} """
        
        if unit == RotationalStiffnessPerLengthUnits.MeganewtonMeterPerRadianPerMeter:
            return f"""{self.meganewton_meters_per_radian_per_meter} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: RotationalStiffnessPerLengthUnits = RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter) -> string:
        """
        Get RotationalStiffnessPerLength unit abbreviation.
        Note! the default abbreviation for RotationalStiffnessPerLength is NewtonMeterPerRadianPerMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RotationalStiffnessPerLengthUnits.NewtonMeterPerRadianPerMeter:
            return """N·m/rad/m"""
        
        if unit_abbreviation == RotationalStiffnessPerLengthUnits.PoundForceFootPerDegreesPerFoot:
            return """lbf·ft/deg/ft"""
        
        if unit_abbreviation == RotationalStiffnessPerLengthUnits.KilopoundForceFootPerDegreesPerFoot:
            return """kipf·ft/°/ft"""
        
        if unit_abbreviation == RotationalStiffnessPerLengthUnits.KilonewtonMeterPerRadianPerMeter:
            return """"""
        
        if unit_abbreviation == RotationalStiffnessPerLengthUnits.MeganewtonMeterPerRadianPerMeter:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, RotationalStiffnessPerLength):
            raise TypeError("unsupported operand type(s) for +: 'RotationalStiffnessPerLength' and '{}'".format(type(other).__name__))
        return RotationalStiffnessPerLength(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, RotationalStiffnessPerLength):
            raise TypeError("unsupported operand type(s) for *: 'RotationalStiffnessPerLength' and '{}'".format(type(other).__name__))
        return RotationalStiffnessPerLength(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, RotationalStiffnessPerLength):
            raise TypeError("unsupported operand type(s) for -: 'RotationalStiffnessPerLength' and '{}'".format(type(other).__name__))
        return RotationalStiffnessPerLength(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, RotationalStiffnessPerLength):
            raise TypeError("unsupported operand type(s) for /: 'RotationalStiffnessPerLength' and '{}'".format(type(other).__name__))
        return RotationalStiffnessPerLength(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, RotationalStiffnessPerLength):
            raise TypeError("unsupported operand type(s) for %: 'RotationalStiffnessPerLength' and '{}'".format(type(other).__name__))
        return RotationalStiffnessPerLength(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, RotationalStiffnessPerLength):
            raise TypeError("unsupported operand type(s) for **: 'RotationalStiffnessPerLength' and '{}'".format(type(other).__name__))
        return RotationalStiffnessPerLength(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, RotationalStiffnessPerLength):
            raise TypeError("unsupported operand type(s) for ==: 'RotationalStiffnessPerLength' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, RotationalStiffnessPerLength):
            raise TypeError("unsupported operand type(s) for <: 'RotationalStiffnessPerLength' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, RotationalStiffnessPerLength):
            raise TypeError("unsupported operand type(s) for >: 'RotationalStiffnessPerLength' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, RotationalStiffnessPerLength):
            raise TypeError("unsupported operand type(s) for <=: 'RotationalStiffnessPerLength' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, RotationalStiffnessPerLength):
            raise TypeError("unsupported operand type(s) for >=: 'RotationalStiffnessPerLength' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class LeakRateUnits(Enum):
        """
            LeakRateUnits enumeration
        """
        
        PascalCubicMeterPerSecond = 'pascal_cubic_meter_per_second'
        """
            
        """
        
        MillibarLiterPerSecond = 'millibar_liter_per_second'
        """
            
        """
        
        TorrLiterPerSecond = 'torr_liter_per_second'
        """
            
        """
        

class LeakRate:
    """
    A leakage rate of QL = 1 Pa-m³/s is given when the pressure in a closed, evacuated container with a volume of 1 m³ rises by 1 Pa per second or when the pressure in the container drops by 1 Pa in the event of overpressure.

    Args:
        value (float): The value.
        from_unit (LeakRateUnits): The LeakRate unit to create from, The default unit is PascalCubicMeterPerSecond
    """
    def __init__(self, value: float, from_unit: LeakRateUnits = LeakRateUnits.PascalCubicMeterPerSecond):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__pascal_cubic_meters_per_second = None
        
        self.__millibar_liters_per_second = None
        
        self.__torr_liters_per_second = None
        

    def __convert_from_base(self, from_unit: LeakRateUnits) -> float:
        value = self.__value
        
        if from_unit == LeakRateUnits.PascalCubicMeterPerSecond:
            return (value)
        
        if from_unit == LeakRateUnits.MillibarLiterPerSecond:
            return (value * 10)
        
        if from_unit == LeakRateUnits.TorrLiterPerSecond:
            return (value * 7.5)
        
        return None


    def __convert_to_base(self, value: float, to_unit: LeakRateUnits) -> float:
        
        if to_unit == LeakRateUnits.PascalCubicMeterPerSecond:
            return (value)
        
        if to_unit == LeakRateUnits.MillibarLiterPerSecond:
            return (value / 10)
        
        if to_unit == LeakRateUnits.TorrLiterPerSecond:
            return (value / 7.5)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_pascal_cubic_meters_per_second(pascal_cubic_meters_per_second: float):
        """
        Create a new instance of LeakRate from a value in pascal_cubic_meters_per_second.

        

        :param meters: The LeakRate value in pascal_cubic_meters_per_second.
        :type pascal_cubic_meters_per_second: float
        :return: A new instance of LeakRate.
        :rtype: LeakRate
        """
        return LeakRate(pascal_cubic_meters_per_second, LeakRateUnits.PascalCubicMeterPerSecond)

    
    @staticmethod
    def from_millibar_liters_per_second(millibar_liters_per_second: float):
        """
        Create a new instance of LeakRate from a value in millibar_liters_per_second.

        

        :param meters: The LeakRate value in millibar_liters_per_second.
        :type millibar_liters_per_second: float
        :return: A new instance of LeakRate.
        :rtype: LeakRate
        """
        return LeakRate(millibar_liters_per_second, LeakRateUnits.MillibarLiterPerSecond)

    
    @staticmethod
    def from_torr_liters_per_second(torr_liters_per_second: float):
        """
        Create a new instance of LeakRate from a value in torr_liters_per_second.

        

        :param meters: The LeakRate value in torr_liters_per_second.
        :type torr_liters_per_second: float
        :return: A new instance of LeakRate.
        :rtype: LeakRate
        """
        return LeakRate(torr_liters_per_second, LeakRateUnits.TorrLiterPerSecond)

    
    @property
    def pascal_cubic_meters_per_second(self) -> float:
        """
        
        """
        if self.__pascal_cubic_meters_per_second != None:
            return self.__pascal_cubic_meters_per_second
        self.__pascal_cubic_meters_per_second = self.__convert_from_base(LeakRateUnits.PascalCubicMeterPerSecond)
        return self.__pascal_cubic_meters_per_second

    
    @property
    def millibar_liters_per_second(self) -> float:
        """
        
        """
        if self.__millibar_liters_per_second != None:
            return self.__millibar_liters_per_second
        self.__millibar_liters_per_second = self.__convert_from_base(LeakRateUnits.MillibarLiterPerSecond)
        return self.__millibar_liters_per_second

    
    @property
    def torr_liters_per_second(self) -> float:
        """
        
        """
        if self.__torr_liters_per_second != None:
            return self.__torr_liters_per_second
        self.__torr_liters_per_second = self.__convert_from_base(LeakRateUnits.TorrLiterPerSecond)
        return self.__torr_liters_per_second

    
    def to_string(self, unit: LeakRateUnits = LeakRateUnits.PascalCubicMeterPerSecond) -> string:
        """
        Format the LeakRate to string.
        Note! the default format for LeakRate is PascalCubicMeterPerSecond.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == LeakRateUnits.PascalCubicMeterPerSecond:
            return f"""{self.pascal_cubic_meters_per_second} Pa·m³/s"""
        
        if unit == LeakRateUnits.MillibarLiterPerSecond:
            return f"""{self.millibar_liters_per_second} mbar·l/s"""
        
        if unit == LeakRateUnits.TorrLiterPerSecond:
            return f"""{self.torr_liters_per_second} Torr·l/s"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: LeakRateUnits = LeakRateUnits.PascalCubicMeterPerSecond) -> string:
        """
        Get LeakRate unit abbreviation.
        Note! the default abbreviation for LeakRate is PascalCubicMeterPerSecond.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == LeakRateUnits.PascalCubicMeterPerSecond:
            return """Pa·m³/s"""
        
        if unit_abbreviation == LeakRateUnits.MillibarLiterPerSecond:
            return """mbar·l/s"""
        
        if unit_abbreviation == LeakRateUnits.TorrLiterPerSecond:
            return """Torr·l/s"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, LeakRate):
            raise TypeError("unsupported operand type(s) for +: 'LeakRate' and '{}'".format(type(other).__name__))
        return LeakRate(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, LeakRate):
            raise TypeError("unsupported operand type(s) for *: 'LeakRate' and '{}'".format(type(other).__name__))
        return LeakRate(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, LeakRate):
            raise TypeError("unsupported operand type(s) for -: 'LeakRate' and '{}'".format(type(other).__name__))
        return LeakRate(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, LeakRate):
            raise TypeError("unsupported operand type(s) for /: 'LeakRate' and '{}'".format(type(other).__name__))
        return LeakRate(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, LeakRate):
            raise TypeError("unsupported operand type(s) for %: 'LeakRate' and '{}'".format(type(other).__name__))
        return LeakRate(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, LeakRate):
            raise TypeError("unsupported operand type(s) for **: 'LeakRate' and '{}'".format(type(other).__name__))
        return LeakRate(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, LeakRate):
            raise TypeError("unsupported operand type(s) for ==: 'LeakRate' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, LeakRate):
            raise TypeError("unsupported operand type(s) for <: 'LeakRate' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, LeakRate):
            raise TypeError("unsupported operand type(s) for >: 'LeakRate' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, LeakRate):
            raise TypeError("unsupported operand type(s) for <=: 'LeakRate' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, LeakRate):
            raise TypeError("unsupported operand type(s) for >=: 'LeakRate' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class MolarEntropyUnits(Enum):
        """
            MolarEntropyUnits enumeration
        """
        
        JoulePerMoleKelvin = 'joule_per_mole_kelvin'
        """
            
        """
        
        KilojoulePerMoleKelvin = 'kilojoule_per_mole_kelvin'
        """
            
        """
        
        MegajoulePerMoleKelvin = 'megajoule_per_mole_kelvin'
        """
            
        """
        

class MolarEntropy:
    """
    Molar entropy is amount of energy required to increase temperature of 1 mole substance by 1 Kelvin.

    Args:
        value (float): The value.
        from_unit (MolarEntropyUnits): The MolarEntropy unit to create from, The default unit is JoulePerMoleKelvin
    """
    def __init__(self, value: float, from_unit: MolarEntropyUnits = MolarEntropyUnits.JoulePerMoleKelvin):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_mole_kelvin = None
        
        self.__kilojoules_per_mole_kelvin = None
        
        self.__megajoules_per_mole_kelvin = None
        

    def __convert_from_base(self, from_unit: MolarEntropyUnits) -> float:
        value = self.__value
        
        if from_unit == MolarEntropyUnits.JoulePerMoleKelvin:
            return (value)
        
        if from_unit == MolarEntropyUnits.KilojoulePerMoleKelvin:
            return ((value) / 1000.0)
        
        if from_unit == MolarEntropyUnits.MegajoulePerMoleKelvin:
            return ((value) / 1000000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MolarEntropyUnits) -> float:
        
        if to_unit == MolarEntropyUnits.JoulePerMoleKelvin:
            return (value)
        
        if to_unit == MolarEntropyUnits.KilojoulePerMoleKelvin:
            return ((value) * 1000.0)
        
        if to_unit == MolarEntropyUnits.MegajoulePerMoleKelvin:
            return ((value) * 1000000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_joules_per_mole_kelvin(joules_per_mole_kelvin: float):
        """
        Create a new instance of MolarEntropy from a value in joules_per_mole_kelvin.

        

        :param meters: The MolarEntropy value in joules_per_mole_kelvin.
        :type joules_per_mole_kelvin: float
        :return: A new instance of MolarEntropy.
        :rtype: MolarEntropy
        """
        return MolarEntropy(joules_per_mole_kelvin, MolarEntropyUnits.JoulePerMoleKelvin)

    
    @staticmethod
    def from_kilojoules_per_mole_kelvin(kilojoules_per_mole_kelvin: float):
        """
        Create a new instance of MolarEntropy from a value in kilojoules_per_mole_kelvin.

        

        :param meters: The MolarEntropy value in kilojoules_per_mole_kelvin.
        :type kilojoules_per_mole_kelvin: float
        :return: A new instance of MolarEntropy.
        :rtype: MolarEntropy
        """
        return MolarEntropy(kilojoules_per_mole_kelvin, MolarEntropyUnits.KilojoulePerMoleKelvin)

    
    @staticmethod
    def from_megajoules_per_mole_kelvin(megajoules_per_mole_kelvin: float):
        """
        Create a new instance of MolarEntropy from a value in megajoules_per_mole_kelvin.

        

        :param meters: The MolarEntropy value in megajoules_per_mole_kelvin.
        :type megajoules_per_mole_kelvin: float
        :return: A new instance of MolarEntropy.
        :rtype: MolarEntropy
        """
        return MolarEntropy(megajoules_per_mole_kelvin, MolarEntropyUnits.MegajoulePerMoleKelvin)

    
    @property
    def joules_per_mole_kelvin(self) -> float:
        """
        
        """
        if self.__joules_per_mole_kelvin != None:
            return self.__joules_per_mole_kelvin
        self.__joules_per_mole_kelvin = self.__convert_from_base(MolarEntropyUnits.JoulePerMoleKelvin)
        return self.__joules_per_mole_kelvin

    
    @property
    def kilojoules_per_mole_kelvin(self) -> float:
        """
        
        """
        if self.__kilojoules_per_mole_kelvin != None:
            return self.__kilojoules_per_mole_kelvin
        self.__kilojoules_per_mole_kelvin = self.__convert_from_base(MolarEntropyUnits.KilojoulePerMoleKelvin)
        return self.__kilojoules_per_mole_kelvin

    
    @property
    def megajoules_per_mole_kelvin(self) -> float:
        """
        
        """
        if self.__megajoules_per_mole_kelvin != None:
            return self.__megajoules_per_mole_kelvin
        self.__megajoules_per_mole_kelvin = self.__convert_from_base(MolarEntropyUnits.MegajoulePerMoleKelvin)
        return self.__megajoules_per_mole_kelvin

    
    def to_string(self, unit: MolarEntropyUnits = MolarEntropyUnits.JoulePerMoleKelvin) -> string:
        """
        Format the MolarEntropy to string.
        Note! the default format for MolarEntropy is JoulePerMoleKelvin.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == MolarEntropyUnits.JoulePerMoleKelvin:
            return f"""{self.joules_per_mole_kelvin} J/(mol*K)"""
        
        if unit == MolarEntropyUnits.KilojoulePerMoleKelvin:
            return f"""{self.kilojoules_per_mole_kelvin} """
        
        if unit == MolarEntropyUnits.MegajoulePerMoleKelvin:
            return f"""{self.megajoules_per_mole_kelvin} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: MolarEntropyUnits = MolarEntropyUnits.JoulePerMoleKelvin) -> string:
        """
        Get MolarEntropy unit abbreviation.
        Note! the default abbreviation for MolarEntropy is JoulePerMoleKelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MolarEntropyUnits.JoulePerMoleKelvin:
            return """J/(mol*K)"""
        
        if unit_abbreviation == MolarEntropyUnits.KilojoulePerMoleKelvin:
            return """"""
        
        if unit_abbreviation == MolarEntropyUnits.MegajoulePerMoleKelvin:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, MolarEntropy):
            raise TypeError("unsupported operand type(s) for +: 'MolarEntropy' and '{}'".format(type(other).__name__))
        return MolarEntropy(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, MolarEntropy):
            raise TypeError("unsupported operand type(s) for *: 'MolarEntropy' and '{}'".format(type(other).__name__))
        return MolarEntropy(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, MolarEntropy):
            raise TypeError("unsupported operand type(s) for -: 'MolarEntropy' and '{}'".format(type(other).__name__))
        return MolarEntropy(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, MolarEntropy):
            raise TypeError("unsupported operand type(s) for /: 'MolarEntropy' and '{}'".format(type(other).__name__))
        return MolarEntropy(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, MolarEntropy):
            raise TypeError("unsupported operand type(s) for %: 'MolarEntropy' and '{}'".format(type(other).__name__))
        return MolarEntropy(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, MolarEntropy):
            raise TypeError("unsupported operand type(s) for **: 'MolarEntropy' and '{}'".format(type(other).__name__))
        return MolarEntropy(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, MolarEntropy):
            raise TypeError("unsupported operand type(s) for ==: 'MolarEntropy' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, MolarEntropy):
            raise TypeError("unsupported operand type(s) for <: 'MolarEntropy' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, MolarEntropy):
            raise TypeError("unsupported operand type(s) for >: 'MolarEntropy' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, MolarEntropy):
            raise TypeError("unsupported operand type(s) for <=: 'MolarEntropy' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, MolarEntropy):
            raise TypeError("unsupported operand type(s) for >=: 'MolarEntropy' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class MolarityUnits(Enum):
        """
            MolarityUnits enumeration
        """
        
        MolePerCubicMeter = 'mole_per_cubic_meter'
        """
            
        """
        
        MolePerLiter = 'mole_per_liter'
        """
            
        """
        
        PoundMolePerCubicFoot = 'pound_mole_per_cubic_foot'
        """
            
        """
        
        KilomolePerCubicMeter = 'kilomole_per_cubic_meter'
        """
            
        """
        
        FemtomolePerLiter = 'femtomole_per_liter'
        """
            
        """
        
        PicomolePerLiter = 'picomole_per_liter'
        """
            
        """
        
        NanomolePerLiter = 'nanomole_per_liter'
        """
            
        """
        
        MicromolePerLiter = 'micromole_per_liter'
        """
            
        """
        
        MillimolePerLiter = 'millimole_per_liter'
        """
            
        """
        
        CentimolePerLiter = 'centimole_per_liter'
        """
            
        """
        
        DecimolePerLiter = 'decimole_per_liter'
        """
            
        """
        

class Molarity:
    """
    Molar concentration, also called molarity, amount concentration or substance concentration, is a measure of the concentration of a solute in a solution, or of any chemical species, in terms of amount of substance in a given volume. 

    Args:
        value (float): The value.
        from_unit (MolarityUnits): The Molarity unit to create from, The default unit is MolePerCubicMeter
    """
    def __init__(self, value: float, from_unit: MolarityUnits = MolarityUnits.MolePerCubicMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__moles_per_cubic_meter = None
        
        self.__moles_per_liter = None
        
        self.__pound_moles_per_cubic_foot = None
        
        self.__kilomoles_per_cubic_meter = None
        
        self.__femtomoles_per_liter = None
        
        self.__picomoles_per_liter = None
        
        self.__nanomoles_per_liter = None
        
        self.__micromoles_per_liter = None
        
        self.__millimoles_per_liter = None
        
        self.__centimoles_per_liter = None
        
        self.__decimoles_per_liter = None
        

    def __convert_from_base(self, from_unit: MolarityUnits) -> float:
        value = self.__value
        
        if from_unit == MolarityUnits.MolePerCubicMeter:
            return (value)
        
        if from_unit == MolarityUnits.MolePerLiter:
            return (value * 1e-3)
        
        if from_unit == MolarityUnits.PoundMolePerCubicFoot:
            return (value * 6.2427960576144611956325455827221e-5)
        
        if from_unit == MolarityUnits.KilomolePerCubicMeter:
            return ((value) / 1000.0)
        
        if from_unit == MolarityUnits.FemtomolePerLiter:
            return ((value * 1e-3) / 1e-15)
        
        if from_unit == MolarityUnits.PicomolePerLiter:
            return ((value * 1e-3) / 1e-12)
        
        if from_unit == MolarityUnits.NanomolePerLiter:
            return ((value * 1e-3) / 1e-09)
        
        if from_unit == MolarityUnits.MicromolePerLiter:
            return ((value * 1e-3) / 1e-06)
        
        if from_unit == MolarityUnits.MillimolePerLiter:
            return ((value * 1e-3) / 0.001)
        
        if from_unit == MolarityUnits.CentimolePerLiter:
            return ((value * 1e-3) / 0.01)
        
        if from_unit == MolarityUnits.DecimolePerLiter:
            return ((value * 1e-3) / 0.1)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MolarityUnits) -> float:
        
        if to_unit == MolarityUnits.MolePerCubicMeter:
            return (value)
        
        if to_unit == MolarityUnits.MolePerLiter:
            return (value / 1e-3)
        
        if to_unit == MolarityUnits.PoundMolePerCubicFoot:
            return (value / 6.2427960576144611956325455827221e-5)
        
        if to_unit == MolarityUnits.KilomolePerCubicMeter:
            return ((value) * 1000.0)
        
        if to_unit == MolarityUnits.FemtomolePerLiter:
            return ((value / 1e-3) * 1e-15)
        
        if to_unit == MolarityUnits.PicomolePerLiter:
            return ((value / 1e-3) * 1e-12)
        
        if to_unit == MolarityUnits.NanomolePerLiter:
            return ((value / 1e-3) * 1e-09)
        
        if to_unit == MolarityUnits.MicromolePerLiter:
            return ((value / 1e-3) * 1e-06)
        
        if to_unit == MolarityUnits.MillimolePerLiter:
            return ((value / 1e-3) * 0.001)
        
        if to_unit == MolarityUnits.CentimolePerLiter:
            return ((value / 1e-3) * 0.01)
        
        if to_unit == MolarityUnits.DecimolePerLiter:
            return ((value / 1e-3) * 0.1)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_moles_per_cubic_meter(moles_per_cubic_meter: float):
        """
        Create a new instance of Molarity from a value in moles_per_cubic_meter.

        

        :param meters: The Molarity value in moles_per_cubic_meter.
        :type moles_per_cubic_meter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(moles_per_cubic_meter, MolarityUnits.MolePerCubicMeter)

    
    @staticmethod
    def from_moles_per_liter(moles_per_liter: float):
        """
        Create a new instance of Molarity from a value in moles_per_liter.

        

        :param meters: The Molarity value in moles_per_liter.
        :type moles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(moles_per_liter, MolarityUnits.MolePerLiter)

    
    @staticmethod
    def from_pound_moles_per_cubic_foot(pound_moles_per_cubic_foot: float):
        """
        Create a new instance of Molarity from a value in pound_moles_per_cubic_foot.

        

        :param meters: The Molarity value in pound_moles_per_cubic_foot.
        :type pound_moles_per_cubic_foot: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(pound_moles_per_cubic_foot, MolarityUnits.PoundMolePerCubicFoot)

    
    @staticmethod
    def from_kilomoles_per_cubic_meter(kilomoles_per_cubic_meter: float):
        """
        Create a new instance of Molarity from a value in kilomoles_per_cubic_meter.

        

        :param meters: The Molarity value in kilomoles_per_cubic_meter.
        :type kilomoles_per_cubic_meter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(kilomoles_per_cubic_meter, MolarityUnits.KilomolePerCubicMeter)

    
    @staticmethod
    def from_femtomoles_per_liter(femtomoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in femtomoles_per_liter.

        

        :param meters: The Molarity value in femtomoles_per_liter.
        :type femtomoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(femtomoles_per_liter, MolarityUnits.FemtomolePerLiter)

    
    @staticmethod
    def from_picomoles_per_liter(picomoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in picomoles_per_liter.

        

        :param meters: The Molarity value in picomoles_per_liter.
        :type picomoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(picomoles_per_liter, MolarityUnits.PicomolePerLiter)

    
    @staticmethod
    def from_nanomoles_per_liter(nanomoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in nanomoles_per_liter.

        

        :param meters: The Molarity value in nanomoles_per_liter.
        :type nanomoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(nanomoles_per_liter, MolarityUnits.NanomolePerLiter)

    
    @staticmethod
    def from_micromoles_per_liter(micromoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in micromoles_per_liter.

        

        :param meters: The Molarity value in micromoles_per_liter.
        :type micromoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(micromoles_per_liter, MolarityUnits.MicromolePerLiter)

    
    @staticmethod
    def from_millimoles_per_liter(millimoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in millimoles_per_liter.

        

        :param meters: The Molarity value in millimoles_per_liter.
        :type millimoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(millimoles_per_liter, MolarityUnits.MillimolePerLiter)

    
    @staticmethod
    def from_centimoles_per_liter(centimoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in centimoles_per_liter.

        

        :param meters: The Molarity value in centimoles_per_liter.
        :type centimoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(centimoles_per_liter, MolarityUnits.CentimolePerLiter)

    
    @staticmethod
    def from_decimoles_per_liter(decimoles_per_liter: float):
        """
        Create a new instance of Molarity from a value in decimoles_per_liter.

        

        :param meters: The Molarity value in decimoles_per_liter.
        :type decimoles_per_liter: float
        :return: A new instance of Molarity.
        :rtype: Molarity
        """
        return Molarity(decimoles_per_liter, MolarityUnits.DecimolePerLiter)

    
    @property
    def moles_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__moles_per_cubic_meter != None:
            return self.__moles_per_cubic_meter
        self.__moles_per_cubic_meter = self.__convert_from_base(MolarityUnits.MolePerCubicMeter)
        return self.__moles_per_cubic_meter

    
    @property
    def moles_per_liter(self) -> float:
        """
        
        """
        if self.__moles_per_liter != None:
            return self.__moles_per_liter
        self.__moles_per_liter = self.__convert_from_base(MolarityUnits.MolePerLiter)
        return self.__moles_per_liter

    
    @property
    def pound_moles_per_cubic_foot(self) -> float:
        """
        
        """
        if self.__pound_moles_per_cubic_foot != None:
            return self.__pound_moles_per_cubic_foot
        self.__pound_moles_per_cubic_foot = self.__convert_from_base(MolarityUnits.PoundMolePerCubicFoot)
        return self.__pound_moles_per_cubic_foot

    
    @property
    def kilomoles_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__kilomoles_per_cubic_meter != None:
            return self.__kilomoles_per_cubic_meter
        self.__kilomoles_per_cubic_meter = self.__convert_from_base(MolarityUnits.KilomolePerCubicMeter)
        return self.__kilomoles_per_cubic_meter

    
    @property
    def femtomoles_per_liter(self) -> float:
        """
        
        """
        if self.__femtomoles_per_liter != None:
            return self.__femtomoles_per_liter
        self.__femtomoles_per_liter = self.__convert_from_base(MolarityUnits.FemtomolePerLiter)
        return self.__femtomoles_per_liter

    
    @property
    def picomoles_per_liter(self) -> float:
        """
        
        """
        if self.__picomoles_per_liter != None:
            return self.__picomoles_per_liter
        self.__picomoles_per_liter = self.__convert_from_base(MolarityUnits.PicomolePerLiter)
        return self.__picomoles_per_liter

    
    @property
    def nanomoles_per_liter(self) -> float:
        """
        
        """
        if self.__nanomoles_per_liter != None:
            return self.__nanomoles_per_liter
        self.__nanomoles_per_liter = self.__convert_from_base(MolarityUnits.NanomolePerLiter)
        return self.__nanomoles_per_liter

    
    @property
    def micromoles_per_liter(self) -> float:
        """
        
        """
        if self.__micromoles_per_liter != None:
            return self.__micromoles_per_liter
        self.__micromoles_per_liter = self.__convert_from_base(MolarityUnits.MicromolePerLiter)
        return self.__micromoles_per_liter

    
    @property
    def millimoles_per_liter(self) -> float:
        """
        
        """
        if self.__millimoles_per_liter != None:
            return self.__millimoles_per_liter
        self.__millimoles_per_liter = self.__convert_from_base(MolarityUnits.MillimolePerLiter)
        return self.__millimoles_per_liter

    
    @property
    def centimoles_per_liter(self) -> float:
        """
        
        """
        if self.__centimoles_per_liter != None:
            return self.__centimoles_per_liter
        self.__centimoles_per_liter = self.__convert_from_base(MolarityUnits.CentimolePerLiter)
        return self.__centimoles_per_liter

    
    @property
    def decimoles_per_liter(self) -> float:
        """
        
        """
        if self.__decimoles_per_liter != None:
            return self.__decimoles_per_liter
        self.__decimoles_per_liter = self.__convert_from_base(MolarityUnits.DecimolePerLiter)
        return self.__decimoles_per_liter

    
    def to_string(self, unit: MolarityUnits = MolarityUnits.MolePerCubicMeter) -> string:
        """
        Format the Molarity to string.
        Note! the default format for Molarity is MolePerCubicMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == MolarityUnits.MolePerCubicMeter:
            return f"""{self.moles_per_cubic_meter} mol/m³"""
        
        if unit == MolarityUnits.MolePerLiter:
            return f"""{self.moles_per_liter} mol/L"""
        
        if unit == MolarityUnits.PoundMolePerCubicFoot:
            return f"""{self.pound_moles_per_cubic_foot} lbmol/ft³"""
        
        if unit == MolarityUnits.KilomolePerCubicMeter:
            return f"""{self.kilomoles_per_cubic_meter} """
        
        if unit == MolarityUnits.FemtomolePerLiter:
            return f"""{self.femtomoles_per_liter} """
        
        if unit == MolarityUnits.PicomolePerLiter:
            return f"""{self.picomoles_per_liter} """
        
        if unit == MolarityUnits.NanomolePerLiter:
            return f"""{self.nanomoles_per_liter} """
        
        if unit == MolarityUnits.MicromolePerLiter:
            return f"""{self.micromoles_per_liter} """
        
        if unit == MolarityUnits.MillimolePerLiter:
            return f"""{self.millimoles_per_liter} """
        
        if unit == MolarityUnits.CentimolePerLiter:
            return f"""{self.centimoles_per_liter} """
        
        if unit == MolarityUnits.DecimolePerLiter:
            return f"""{self.decimoles_per_liter} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: MolarityUnits = MolarityUnits.MolePerCubicMeter) -> string:
        """
        Get Molarity unit abbreviation.
        Note! the default abbreviation for Molarity is MolePerCubicMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MolarityUnits.MolePerCubicMeter:
            return """mol/m³"""
        
        if unit_abbreviation == MolarityUnits.MolePerLiter:
            return """mol/L"""
        
        if unit_abbreviation == MolarityUnits.PoundMolePerCubicFoot:
            return """lbmol/ft³"""
        
        if unit_abbreviation == MolarityUnits.KilomolePerCubicMeter:
            return """"""
        
        if unit_abbreviation == MolarityUnits.FemtomolePerLiter:
            return """"""
        
        if unit_abbreviation == MolarityUnits.PicomolePerLiter:
            return """"""
        
        if unit_abbreviation == MolarityUnits.NanomolePerLiter:
            return """"""
        
        if unit_abbreviation == MolarityUnits.MicromolePerLiter:
            return """"""
        
        if unit_abbreviation == MolarityUnits.MillimolePerLiter:
            return """"""
        
        if unit_abbreviation == MolarityUnits.CentimolePerLiter:
            return """"""
        
        if unit_abbreviation == MolarityUnits.DecimolePerLiter:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Molarity):
            raise TypeError("unsupported operand type(s) for +: 'Molarity' and '{}'".format(type(other).__name__))
        return Molarity(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Molarity):
            raise TypeError("unsupported operand type(s) for *: 'Molarity' and '{}'".format(type(other).__name__))
        return Molarity(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Molarity):
            raise TypeError("unsupported operand type(s) for -: 'Molarity' and '{}'".format(type(other).__name__))
        return Molarity(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Molarity):
            raise TypeError("unsupported operand type(s) for /: 'Molarity' and '{}'".format(type(other).__name__))
        return Molarity(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Molarity):
            raise TypeError("unsupported operand type(s) for %: 'Molarity' and '{}'".format(type(other).__name__))
        return Molarity(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Molarity):
            raise TypeError("unsupported operand type(s) for **: 'Molarity' and '{}'".format(type(other).__name__))
        return Molarity(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Molarity):
            raise TypeError("unsupported operand type(s) for ==: 'Molarity' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Molarity):
            raise TypeError("unsupported operand type(s) for <: 'Molarity' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Molarity):
            raise TypeError("unsupported operand type(s) for >: 'Molarity' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Molarity):
            raise TypeError("unsupported operand type(s) for <=: 'Molarity' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Molarity):
            raise TypeError("unsupported operand type(s) for >=: 'Molarity' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class ThermalConductivityUnits(Enum):
        """
            ThermalConductivityUnits enumeration
        """
        
        WattPerMeterKelvin = 'watt_per_meter_kelvin'
        """
            
        """
        
        BtuPerHourFootFahrenheit = 'btu_per_hour_foot_fahrenheit'
        """
            
        """
        

class ThermalConductivity:
    """
    Thermal conductivity is the property of a material to conduct heat.

    Args:
        value (float): The value.
        from_unit (ThermalConductivityUnits): The ThermalConductivity unit to create from, The default unit is WattPerMeterKelvin
    """
    def __init__(self, value: float, from_unit: ThermalConductivityUnits = ThermalConductivityUnits.WattPerMeterKelvin):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__watts_per_meter_kelvin = None
        
        self.__btus_per_hour_foot_fahrenheit = None
        

    def __convert_from_base(self, from_unit: ThermalConductivityUnits) -> float:
        value = self.__value
        
        if from_unit == ThermalConductivityUnits.WattPerMeterKelvin:
            return (value)
        
        if from_unit == ThermalConductivityUnits.BtuPerHourFootFahrenheit:
            return (value / 1.73073467)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ThermalConductivityUnits) -> float:
        
        if to_unit == ThermalConductivityUnits.WattPerMeterKelvin:
            return (value)
        
        if to_unit == ThermalConductivityUnits.BtuPerHourFootFahrenheit:
            return (value * 1.73073467)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_watts_per_meter_kelvin(watts_per_meter_kelvin: float):
        """
        Create a new instance of ThermalConductivity from a value in watts_per_meter_kelvin.

        

        :param meters: The ThermalConductivity value in watts_per_meter_kelvin.
        :type watts_per_meter_kelvin: float
        :return: A new instance of ThermalConductivity.
        :rtype: ThermalConductivity
        """
        return ThermalConductivity(watts_per_meter_kelvin, ThermalConductivityUnits.WattPerMeterKelvin)

    
    @staticmethod
    def from_btus_per_hour_foot_fahrenheit(btus_per_hour_foot_fahrenheit: float):
        """
        Create a new instance of ThermalConductivity from a value in btus_per_hour_foot_fahrenheit.

        

        :param meters: The ThermalConductivity value in btus_per_hour_foot_fahrenheit.
        :type btus_per_hour_foot_fahrenheit: float
        :return: A new instance of ThermalConductivity.
        :rtype: ThermalConductivity
        """
        return ThermalConductivity(btus_per_hour_foot_fahrenheit, ThermalConductivityUnits.BtuPerHourFootFahrenheit)

    
    @property
    def watts_per_meter_kelvin(self) -> float:
        """
        
        """
        if self.__watts_per_meter_kelvin != None:
            return self.__watts_per_meter_kelvin
        self.__watts_per_meter_kelvin = self.__convert_from_base(ThermalConductivityUnits.WattPerMeterKelvin)
        return self.__watts_per_meter_kelvin

    
    @property
    def btus_per_hour_foot_fahrenheit(self) -> float:
        """
        
        """
        if self.__btus_per_hour_foot_fahrenheit != None:
            return self.__btus_per_hour_foot_fahrenheit
        self.__btus_per_hour_foot_fahrenheit = self.__convert_from_base(ThermalConductivityUnits.BtuPerHourFootFahrenheit)
        return self.__btus_per_hour_foot_fahrenheit

    
    def to_string(self, unit: ThermalConductivityUnits = ThermalConductivityUnits.WattPerMeterKelvin) -> string:
        """
        Format the ThermalConductivity to string.
        Note! the default format for ThermalConductivity is WattPerMeterKelvin.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ThermalConductivityUnits.WattPerMeterKelvin:
            return f"""{self.watts_per_meter_kelvin} W/m·K"""
        
        if unit == ThermalConductivityUnits.BtuPerHourFootFahrenheit:
            return f"""{self.btus_per_hour_foot_fahrenheit} BTU/h·ft·°F"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: ThermalConductivityUnits = ThermalConductivityUnits.WattPerMeterKelvin) -> string:
        """
        Get ThermalConductivity unit abbreviation.
        Note! the default abbreviation for ThermalConductivity is WattPerMeterKelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ThermalConductivityUnits.WattPerMeterKelvin:
            return """W/m·K"""
        
        if unit_abbreviation == ThermalConductivityUnits.BtuPerHourFootFahrenheit:
            return """BTU/h·ft·°F"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, ThermalConductivity):
            raise TypeError("unsupported operand type(s) for +: 'ThermalConductivity' and '{}'".format(type(other).__name__))
        return ThermalConductivity(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, ThermalConductivity):
            raise TypeError("unsupported operand type(s) for *: 'ThermalConductivity' and '{}'".format(type(other).__name__))
        return ThermalConductivity(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, ThermalConductivity):
            raise TypeError("unsupported operand type(s) for -: 'ThermalConductivity' and '{}'".format(type(other).__name__))
        return ThermalConductivity(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, ThermalConductivity):
            raise TypeError("unsupported operand type(s) for /: 'ThermalConductivity' and '{}'".format(type(other).__name__))
        return ThermalConductivity(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, ThermalConductivity):
            raise TypeError("unsupported operand type(s) for %: 'ThermalConductivity' and '{}'".format(type(other).__name__))
        return ThermalConductivity(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, ThermalConductivity):
            raise TypeError("unsupported operand type(s) for **: 'ThermalConductivity' and '{}'".format(type(other).__name__))
        return ThermalConductivity(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, ThermalConductivity):
            raise TypeError("unsupported operand type(s) for ==: 'ThermalConductivity' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, ThermalConductivity):
            raise TypeError("unsupported operand type(s) for <: 'ThermalConductivity' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, ThermalConductivity):
            raise TypeError("unsupported operand type(s) for >: 'ThermalConductivity' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, ThermalConductivity):
            raise TypeError("unsupported operand type(s) for <=: 'ThermalConductivity' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, ThermalConductivity):
            raise TypeError("unsupported operand type(s) for >=: 'ThermalConductivity' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class VolumeFlowPerAreaUnits(Enum):
        """
            VolumeFlowPerAreaUnits enumeration
        """
        
        CubicMeterPerSecondPerSquareMeter = 'cubic_meter_per_second_per_square_meter'
        """
            
        """
        
        CubicFootPerMinutePerSquareFoot = 'cubic_foot_per_minute_per_square_foot'
        """
            
        """
        

class VolumeFlowPerArea:
    """
    None

    Args:
        value (float): The value.
        from_unit (VolumeFlowPerAreaUnits): The VolumeFlowPerArea unit to create from, The default unit is CubicMeterPerSecondPerSquareMeter
    """
    def __init__(self, value: float, from_unit: VolumeFlowPerAreaUnits = VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__cubic_meters_per_second_per_square_meter = None
        
        self.__cubic_feet_per_minute_per_square_foot = None
        

    def __convert_from_base(self, from_unit: VolumeFlowPerAreaUnits) -> float:
        value = self.__value
        
        if from_unit == VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter:
            return (value)
        
        if from_unit == VolumeFlowPerAreaUnits.CubicFootPerMinutePerSquareFoot:
            return (value * 196.850394)
        
        return None


    def __convert_to_base(self, value: float, to_unit: VolumeFlowPerAreaUnits) -> float:
        
        if to_unit == VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter:
            return (value)
        
        if to_unit == VolumeFlowPerAreaUnits.CubicFootPerMinutePerSquareFoot:
            return (value / 196.850394)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_cubic_meters_per_second_per_square_meter(cubic_meters_per_second_per_square_meter: float):
        """
        Create a new instance of VolumeFlowPerArea from a value in cubic_meters_per_second_per_square_meter.

        

        :param meters: The VolumeFlowPerArea value in cubic_meters_per_second_per_square_meter.
        :type cubic_meters_per_second_per_square_meter: float
        :return: A new instance of VolumeFlowPerArea.
        :rtype: VolumeFlowPerArea
        """
        return VolumeFlowPerArea(cubic_meters_per_second_per_square_meter, VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter)

    
    @staticmethod
    def from_cubic_feet_per_minute_per_square_foot(cubic_feet_per_minute_per_square_foot: float):
        """
        Create a new instance of VolumeFlowPerArea from a value in cubic_feet_per_minute_per_square_foot.

        

        :param meters: The VolumeFlowPerArea value in cubic_feet_per_minute_per_square_foot.
        :type cubic_feet_per_minute_per_square_foot: float
        :return: A new instance of VolumeFlowPerArea.
        :rtype: VolumeFlowPerArea
        """
        return VolumeFlowPerArea(cubic_feet_per_minute_per_square_foot, VolumeFlowPerAreaUnits.CubicFootPerMinutePerSquareFoot)

    
    @property
    def cubic_meters_per_second_per_square_meter(self) -> float:
        """
        
        """
        if self.__cubic_meters_per_second_per_square_meter != None:
            return self.__cubic_meters_per_second_per_square_meter
        self.__cubic_meters_per_second_per_square_meter = self.__convert_from_base(VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter)
        return self.__cubic_meters_per_second_per_square_meter

    
    @property
    def cubic_feet_per_minute_per_square_foot(self) -> float:
        """
        
        """
        if self.__cubic_feet_per_minute_per_square_foot != None:
            return self.__cubic_feet_per_minute_per_square_foot
        self.__cubic_feet_per_minute_per_square_foot = self.__convert_from_base(VolumeFlowPerAreaUnits.CubicFootPerMinutePerSquareFoot)
        return self.__cubic_feet_per_minute_per_square_foot

    
    def to_string(self, unit: VolumeFlowPerAreaUnits = VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter) -> string:
        """
        Format the VolumeFlowPerArea to string.
        Note! the default format for VolumeFlowPerArea is CubicMeterPerSecondPerSquareMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter:
            return f"""{self.cubic_meters_per_second_per_square_meter} m³/(s·m²)"""
        
        if unit == VolumeFlowPerAreaUnits.CubicFootPerMinutePerSquareFoot:
            return f"""{self.cubic_feet_per_minute_per_square_foot} CFM/ft²"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: VolumeFlowPerAreaUnits = VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter) -> string:
        """
        Get VolumeFlowPerArea unit abbreviation.
        Note! the default abbreviation for VolumeFlowPerArea is CubicMeterPerSecondPerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter:
            return """m³/(s·m²)"""
        
        if unit_abbreviation == VolumeFlowPerAreaUnits.CubicFootPerMinutePerSquareFoot:
            return """CFM/ft²"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, VolumeFlowPerArea):
            raise TypeError("unsupported operand type(s) for +: 'VolumeFlowPerArea' and '{}'".format(type(other).__name__))
        return VolumeFlowPerArea(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, VolumeFlowPerArea):
            raise TypeError("unsupported operand type(s) for *: 'VolumeFlowPerArea' and '{}'".format(type(other).__name__))
        return VolumeFlowPerArea(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, VolumeFlowPerArea):
            raise TypeError("unsupported operand type(s) for -: 'VolumeFlowPerArea' and '{}'".format(type(other).__name__))
        return VolumeFlowPerArea(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, VolumeFlowPerArea):
            raise TypeError("unsupported operand type(s) for /: 'VolumeFlowPerArea' and '{}'".format(type(other).__name__))
        return VolumeFlowPerArea(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, VolumeFlowPerArea):
            raise TypeError("unsupported operand type(s) for %: 'VolumeFlowPerArea' and '{}'".format(type(other).__name__))
        return VolumeFlowPerArea(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, VolumeFlowPerArea):
            raise TypeError("unsupported operand type(s) for **: 'VolumeFlowPerArea' and '{}'".format(type(other).__name__))
        return VolumeFlowPerArea(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, VolumeFlowPerArea):
            raise TypeError("unsupported operand type(s) for ==: 'VolumeFlowPerArea' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, VolumeFlowPerArea):
            raise TypeError("unsupported operand type(s) for <: 'VolumeFlowPerArea' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, VolumeFlowPerArea):
            raise TypeError("unsupported operand type(s) for >: 'VolumeFlowPerArea' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, VolumeFlowPerArea):
            raise TypeError("unsupported operand type(s) for <=: 'VolumeFlowPerArea' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, VolumeFlowPerArea):
            raise TypeError("unsupported operand type(s) for >=: 'VolumeFlowPerArea' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class RatioUnits(Enum):
        """
            RatioUnits enumeration
        """
        
        DecimalFraction = 'decimal_fraction'
        """
            
        """
        
        Percent = 'percent'
        """
            
        """
        
        PartPerThousand = 'part_per_thousand'
        """
            
        """
        
        PartPerMillion = 'part_per_million'
        """
            
        """
        
        PartPerBillion = 'part_per_billion'
        """
            
        """
        
        PartPerTrillion = 'part_per_trillion'
        """
            
        """
        

class Ratio:
    """
    In mathematics, a ratio is a relationship between two numbers of the same kind (e.g., objects, persons, students, spoonfuls, units of whatever identical dimension), usually expressed as "a to b" or a:b, sometimes expressed arithmetically as a dimensionless quotient of the two that explicitly indicates how many times the first number contains the second (not necessarily an integer).

    Args:
        value (float): The value.
        from_unit (RatioUnits): The Ratio unit to create from, The default unit is DecimalFraction
    """
    def __init__(self, value: float, from_unit: RatioUnits = RatioUnits.DecimalFraction):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__decimal_fractions = None
        
        self.__percent = None
        
        self.__parts_per_thousand = None
        
        self.__parts_per_million = None
        
        self.__parts_per_billion = None
        
        self.__parts_per_trillion = None
        

    def __convert_from_base(self, from_unit: RatioUnits) -> float:
        value = self.__value
        
        if from_unit == RatioUnits.DecimalFraction:
            return (value)
        
        if from_unit == RatioUnits.Percent:
            return (value * 1e2)
        
        if from_unit == RatioUnits.PartPerThousand:
            return (value * 1e3)
        
        if from_unit == RatioUnits.PartPerMillion:
            return (value * 1e6)
        
        if from_unit == RatioUnits.PartPerBillion:
            return (value * 1e9)
        
        if from_unit == RatioUnits.PartPerTrillion:
            return (value * 1e12)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RatioUnits) -> float:
        
        if to_unit == RatioUnits.DecimalFraction:
            return (value)
        
        if to_unit == RatioUnits.Percent:
            return (value / 1e2)
        
        if to_unit == RatioUnits.PartPerThousand:
            return (value / 1e3)
        
        if to_unit == RatioUnits.PartPerMillion:
            return (value / 1e6)
        
        if to_unit == RatioUnits.PartPerBillion:
            return (value / 1e9)
        
        if to_unit == RatioUnits.PartPerTrillion:
            return (value / 1e12)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_decimal_fractions(decimal_fractions: float):
        """
        Create a new instance of Ratio from a value in decimal_fractions.

        

        :param meters: The Ratio value in decimal_fractions.
        :type decimal_fractions: float
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(decimal_fractions, RatioUnits.DecimalFraction)

    
    @staticmethod
    def from_percent(percent: float):
        """
        Create a new instance of Ratio from a value in percent.

        

        :param meters: The Ratio value in percent.
        :type percent: float
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(percent, RatioUnits.Percent)

    
    @staticmethod
    def from_parts_per_thousand(parts_per_thousand: float):
        """
        Create a new instance of Ratio from a value in parts_per_thousand.

        

        :param meters: The Ratio value in parts_per_thousand.
        :type parts_per_thousand: float
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(parts_per_thousand, RatioUnits.PartPerThousand)

    
    @staticmethod
    def from_parts_per_million(parts_per_million: float):
        """
        Create a new instance of Ratio from a value in parts_per_million.

        

        :param meters: The Ratio value in parts_per_million.
        :type parts_per_million: float
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(parts_per_million, RatioUnits.PartPerMillion)

    
    @staticmethod
    def from_parts_per_billion(parts_per_billion: float):
        """
        Create a new instance of Ratio from a value in parts_per_billion.

        

        :param meters: The Ratio value in parts_per_billion.
        :type parts_per_billion: float
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(parts_per_billion, RatioUnits.PartPerBillion)

    
    @staticmethod
    def from_parts_per_trillion(parts_per_trillion: float):
        """
        Create a new instance of Ratio from a value in parts_per_trillion.

        

        :param meters: The Ratio value in parts_per_trillion.
        :type parts_per_trillion: float
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(parts_per_trillion, RatioUnits.PartPerTrillion)

    
    @property
    def decimal_fractions(self) -> float:
        """
        
        """
        if self.__decimal_fractions != None:
            return self.__decimal_fractions
        self.__decimal_fractions = self.__convert_from_base(RatioUnits.DecimalFraction)
        return self.__decimal_fractions

    
    @property
    def percent(self) -> float:
        """
        
        """
        if self.__percent != None:
            return self.__percent
        self.__percent = self.__convert_from_base(RatioUnits.Percent)
        return self.__percent

    
    @property
    def parts_per_thousand(self) -> float:
        """
        
        """
        if self.__parts_per_thousand != None:
            return self.__parts_per_thousand
        self.__parts_per_thousand = self.__convert_from_base(RatioUnits.PartPerThousand)
        return self.__parts_per_thousand

    
    @property
    def parts_per_million(self) -> float:
        """
        
        """
        if self.__parts_per_million != None:
            return self.__parts_per_million
        self.__parts_per_million = self.__convert_from_base(RatioUnits.PartPerMillion)
        return self.__parts_per_million

    
    @property
    def parts_per_billion(self) -> float:
        """
        
        """
        if self.__parts_per_billion != None:
            return self.__parts_per_billion
        self.__parts_per_billion = self.__convert_from_base(RatioUnits.PartPerBillion)
        return self.__parts_per_billion

    
    @property
    def parts_per_trillion(self) -> float:
        """
        
        """
        if self.__parts_per_trillion != None:
            return self.__parts_per_trillion
        self.__parts_per_trillion = self.__convert_from_base(RatioUnits.PartPerTrillion)
        return self.__parts_per_trillion

    
    def to_string(self, unit: RatioUnits = RatioUnits.DecimalFraction) -> string:
        """
        Format the Ratio to string.
        Note! the default format for Ratio is DecimalFraction.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == RatioUnits.DecimalFraction:
            return f"""{self.decimal_fractions} """
        
        if unit == RatioUnits.Percent:
            return f"""{self.percent} %"""
        
        if unit == RatioUnits.PartPerThousand:
            return f"""{self.parts_per_thousand} ‰"""
        
        if unit == RatioUnits.PartPerMillion:
            return f"""{self.parts_per_million} ppm"""
        
        if unit == RatioUnits.PartPerBillion:
            return f"""{self.parts_per_billion} ppb"""
        
        if unit == RatioUnits.PartPerTrillion:
            return f"""{self.parts_per_trillion} ppt"""
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: RatioUnits = RatioUnits.DecimalFraction) -> string:
        """
        Get Ratio unit abbreviation.
        Note! the default abbreviation for Ratio is DecimalFraction.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RatioUnits.DecimalFraction:
            return """"""
        
        if unit_abbreviation == RatioUnits.Percent:
            return """%"""
        
        if unit_abbreviation == RatioUnits.PartPerThousand:
            return """‰"""
        
        if unit_abbreviation == RatioUnits.PartPerMillion:
            return """ppm"""
        
        if unit_abbreviation == RatioUnits.PartPerBillion:
            return """ppb"""
        
        if unit_abbreviation == RatioUnits.PartPerTrillion:
            return """ppt"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Ratio):
            raise TypeError("unsupported operand type(s) for +: 'Ratio' and '{}'".format(type(other).__name__))
        return Ratio(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Ratio):
            raise TypeError("unsupported operand type(s) for *: 'Ratio' and '{}'".format(type(other).__name__))
        return Ratio(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Ratio):
            raise TypeError("unsupported operand type(s) for -: 'Ratio' and '{}'".format(type(other).__name__))
        return Ratio(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Ratio):
            raise TypeError("unsupported operand type(s) for /: 'Ratio' and '{}'".format(type(other).__name__))
        return Ratio(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Ratio):
            raise TypeError("unsupported operand type(s) for %: 'Ratio' and '{}'".format(type(other).__name__))
        return Ratio(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Ratio):
            raise TypeError("unsupported operand type(s) for **: 'Ratio' and '{}'".format(type(other).__name__))
        return Ratio(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Ratio):
            raise TypeError("unsupported operand type(s) for ==: 'Ratio' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Ratio):
            raise TypeError("unsupported operand type(s) for <: 'Ratio' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Ratio):
            raise TypeError("unsupported operand type(s) for >: 'Ratio' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Ratio):
            raise TypeError("unsupported operand type(s) for <=: 'Ratio' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Ratio):
            raise TypeError("unsupported operand type(s) for >=: 'Ratio' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class ForceUnits(Enum):
        """
            ForceUnits enumeration
        """
        
        Dyn = 'dyn'
        """
            
        """
        
        KilogramForce = 'kilogram_force'
        """
            
        """
        
        TonneForce = 'tonne_force'
        """
            
        """
        
        Newton = 'newton'
        """
            
        """
        
        KiloPond = 'kilo_pond'
        """
            
        """
        
        Poundal = 'poundal'
        """
            
        """
        
        PoundForce = 'pound_force'
        """
            
        """
        
        OunceForce = 'ounce_force'
        """
            
        """
        
        ShortTonForce = 'short_ton_force'
        """
            The short ton-force is a unit of force equal to 2,000 pounds-force (907.18474 kgf), that is most commonly used in the United States – known there simply as the ton or US ton.
        """
        
        Micronewton = 'micronewton'
        """
            
        """
        
        Millinewton = 'millinewton'
        """
            
        """
        
        Decanewton = 'decanewton'
        """
            
        """
        
        Kilonewton = 'kilonewton'
        """
            
        """
        
        Meganewton = 'meganewton'
        """
            
        """
        
        KilopoundForce = 'kilopound_force'
        """
            
        """
        

class Force:
    """
    In physics, a force is any influence that causes an object to undergo a certain change, either concerning its movement, direction, or geometrical construction. In other words, a force can cause an object with mass to change its velocity (which includes to begin moving from a state of rest), i.e., to accelerate, or a flexible object to deform, or both. Force can also be described by intuitive concepts such as a push or a pull. A force has both magnitude and direction, making it a vector quantity. It is measured in the SI unit of newtons and represented by the symbol F.

    Args:
        value (float): The value.
        from_unit (ForceUnits): The Force unit to create from, The default unit is Newton
    """
    def __init__(self, value: float, from_unit: ForceUnits = ForceUnits.Newton):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__dyne = None
        
        self.__kilograms_force = None
        
        self.__tonnes_force = None
        
        self.__newtons = None
        
        self.__kilo_ponds = None
        
        self.__poundals = None
        
        self.__pounds_force = None
        
        self.__ounce_force = None
        
        self.__short_tons_force = None
        
        self.__micronewtons = None
        
        self.__millinewtons = None
        
        self.__decanewtons = None
        
        self.__kilonewtons = None
        
        self.__meganewtons = None
        
        self.__kilopounds_force = None
        

    def __convert_from_base(self, from_unit: ForceUnits) -> float:
        value = self.__value
        
        if from_unit == ForceUnits.Dyn:
            return (value * 1e5)
        
        if from_unit == ForceUnits.KilogramForce:
            return (value / 9.80665002864)
        
        if from_unit == ForceUnits.TonneForce:
            return (value / 9.80665002864e3)
        
        if from_unit == ForceUnits.Newton:
            return (value)
        
        if from_unit == ForceUnits.KiloPond:
            return (value / 9.80665002864)
        
        if from_unit == ForceUnits.Poundal:
            return (value / 0.13825502798973041652092282466083)
        
        if from_unit == ForceUnits.PoundForce:
            return (value / 4.4482216152605095551842641431421)
        
        if from_unit == ForceUnits.OunceForce:
            return (value / 2.780138509537812e-1)
        
        if from_unit == ForceUnits.ShortTonForce:
            return (value / 8.896443230521e3)
        
        if from_unit == ForceUnits.Micronewton:
            return ((value) / 1e-06)
        
        if from_unit == ForceUnits.Millinewton:
            return ((value) / 0.001)
        
        if from_unit == ForceUnits.Decanewton:
            return ((value) / 10.0)
        
        if from_unit == ForceUnits.Kilonewton:
            return ((value) / 1000.0)
        
        if from_unit == ForceUnits.Meganewton:
            return ((value) / 1000000.0)
        
        if from_unit == ForceUnits.KilopoundForce:
            return ((value / 4.4482216152605095551842641431421) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ForceUnits) -> float:
        
        if to_unit == ForceUnits.Dyn:
            return (value / 1e5)
        
        if to_unit == ForceUnits.KilogramForce:
            return (value * 9.80665002864)
        
        if to_unit == ForceUnits.TonneForce:
            return (value * 9.80665002864e3)
        
        if to_unit == ForceUnits.Newton:
            return (value)
        
        if to_unit == ForceUnits.KiloPond:
            return (value * 9.80665002864)
        
        if to_unit == ForceUnits.Poundal:
            return (value * 0.13825502798973041652092282466083)
        
        if to_unit == ForceUnits.PoundForce:
            return (value * 4.4482216152605095551842641431421)
        
        if to_unit == ForceUnits.OunceForce:
            return (value * 2.780138509537812e-1)
        
        if to_unit == ForceUnits.ShortTonForce:
            return (value * 8.896443230521e3)
        
        if to_unit == ForceUnits.Micronewton:
            return ((value) * 1e-06)
        
        if to_unit == ForceUnits.Millinewton:
            return ((value) * 0.001)
        
        if to_unit == ForceUnits.Decanewton:
            return ((value) * 10.0)
        
        if to_unit == ForceUnits.Kilonewton:
            return ((value) * 1000.0)
        
        if to_unit == ForceUnits.Meganewton:
            return ((value) * 1000000.0)
        
        if to_unit == ForceUnits.KilopoundForce:
            return ((value * 4.4482216152605095551842641431421) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_dyne(dyne: float):
        """
        Create a new instance of Force from a value in dyne.

        

        :param meters: The Force value in dyne.
        :type dyne: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(dyne, ForceUnits.Dyn)

    
    @staticmethod
    def from_kilograms_force(kilograms_force: float):
        """
        Create a new instance of Force from a value in kilograms_force.

        

        :param meters: The Force value in kilograms_force.
        :type kilograms_force: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(kilograms_force, ForceUnits.KilogramForce)

    
    @staticmethod
    def from_tonnes_force(tonnes_force: float):
        """
        Create a new instance of Force from a value in tonnes_force.

        

        :param meters: The Force value in tonnes_force.
        :type tonnes_force: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(tonnes_force, ForceUnits.TonneForce)

    
    @staticmethod
    def from_newtons(newtons: float):
        """
        Create a new instance of Force from a value in newtons.

        

        :param meters: The Force value in newtons.
        :type newtons: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(newtons, ForceUnits.Newton)

    
    @staticmethod
    def from_kilo_ponds(kilo_ponds: float):
        """
        Create a new instance of Force from a value in kilo_ponds.

        

        :param meters: The Force value in kilo_ponds.
        :type kilo_ponds: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(kilo_ponds, ForceUnits.KiloPond)

    
    @staticmethod
    def from_poundals(poundals: float):
        """
        Create a new instance of Force from a value in poundals.

        

        :param meters: The Force value in poundals.
        :type poundals: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(poundals, ForceUnits.Poundal)

    
    @staticmethod
    def from_pounds_force(pounds_force: float):
        """
        Create a new instance of Force from a value in pounds_force.

        

        :param meters: The Force value in pounds_force.
        :type pounds_force: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(pounds_force, ForceUnits.PoundForce)

    
    @staticmethod
    def from_ounce_force(ounce_force: float):
        """
        Create a new instance of Force from a value in ounce_force.

        

        :param meters: The Force value in ounce_force.
        :type ounce_force: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(ounce_force, ForceUnits.OunceForce)

    
    @staticmethod
    def from_short_tons_force(short_tons_force: float):
        """
        Create a new instance of Force from a value in short_tons_force.

        The short ton-force is a unit of force equal to 2,000 pounds-force (907.18474 kgf), that is most commonly used in the United States – known there simply as the ton or US ton.

        :param meters: The Force value in short_tons_force.
        :type short_tons_force: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(short_tons_force, ForceUnits.ShortTonForce)

    
    @staticmethod
    def from_micronewtons(micronewtons: float):
        """
        Create a new instance of Force from a value in micronewtons.

        

        :param meters: The Force value in micronewtons.
        :type micronewtons: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(micronewtons, ForceUnits.Micronewton)

    
    @staticmethod
    def from_millinewtons(millinewtons: float):
        """
        Create a new instance of Force from a value in millinewtons.

        

        :param meters: The Force value in millinewtons.
        :type millinewtons: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(millinewtons, ForceUnits.Millinewton)

    
    @staticmethod
    def from_decanewtons(decanewtons: float):
        """
        Create a new instance of Force from a value in decanewtons.

        

        :param meters: The Force value in decanewtons.
        :type decanewtons: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(decanewtons, ForceUnits.Decanewton)

    
    @staticmethod
    def from_kilonewtons(kilonewtons: float):
        """
        Create a new instance of Force from a value in kilonewtons.

        

        :param meters: The Force value in kilonewtons.
        :type kilonewtons: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(kilonewtons, ForceUnits.Kilonewton)

    
    @staticmethod
    def from_meganewtons(meganewtons: float):
        """
        Create a new instance of Force from a value in meganewtons.

        

        :param meters: The Force value in meganewtons.
        :type meganewtons: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(meganewtons, ForceUnits.Meganewton)

    
    @staticmethod
    def from_kilopounds_force(kilopounds_force: float):
        """
        Create a new instance of Force from a value in kilopounds_force.

        

        :param meters: The Force value in kilopounds_force.
        :type kilopounds_force: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(kilopounds_force, ForceUnits.KilopoundForce)

    
    @property
    def dyne(self) -> float:
        """
        
        """
        if self.__dyne != None:
            return self.__dyne
        self.__dyne = self.__convert_from_base(ForceUnits.Dyn)
        return self.__dyne

    
    @property
    def kilograms_force(self) -> float:
        """
        
        """
        if self.__kilograms_force != None:
            return self.__kilograms_force
        self.__kilograms_force = self.__convert_from_base(ForceUnits.KilogramForce)
        return self.__kilograms_force

    
    @property
    def tonnes_force(self) -> float:
        """
        
        """
        if self.__tonnes_force != None:
            return self.__tonnes_force
        self.__tonnes_force = self.__convert_from_base(ForceUnits.TonneForce)
        return self.__tonnes_force

    
    @property
    def newtons(self) -> float:
        """
        
        """
        if self.__newtons != None:
            return self.__newtons
        self.__newtons = self.__convert_from_base(ForceUnits.Newton)
        return self.__newtons

    
    @property
    def kilo_ponds(self) -> float:
        """
        
        """
        if self.__kilo_ponds != None:
            return self.__kilo_ponds
        self.__kilo_ponds = self.__convert_from_base(ForceUnits.KiloPond)
        return self.__kilo_ponds

    
    @property
    def poundals(self) -> float:
        """
        
        """
        if self.__poundals != None:
            return self.__poundals
        self.__poundals = self.__convert_from_base(ForceUnits.Poundal)
        return self.__poundals

    
    @property
    def pounds_force(self) -> float:
        """
        
        """
        if self.__pounds_force != None:
            return self.__pounds_force
        self.__pounds_force = self.__convert_from_base(ForceUnits.PoundForce)
        return self.__pounds_force

    
    @property
    def ounce_force(self) -> float:
        """
        
        """
        if self.__ounce_force != None:
            return self.__ounce_force
        self.__ounce_force = self.__convert_from_base(ForceUnits.OunceForce)
        return self.__ounce_force

    
    @property
    def short_tons_force(self) -> float:
        """
        The short ton-force is a unit of force equal to 2,000 pounds-force (907.18474 kgf), that is most commonly used in the United States – known there simply as the ton or US ton.
        """
        if self.__short_tons_force != None:
            return self.__short_tons_force
        self.__short_tons_force = self.__convert_from_base(ForceUnits.ShortTonForce)
        return self.__short_tons_force

    
    @property
    def micronewtons(self) -> float:
        """
        
        """
        if self.__micronewtons != None:
            return self.__micronewtons
        self.__micronewtons = self.__convert_from_base(ForceUnits.Micronewton)
        return self.__micronewtons

    
    @property
    def millinewtons(self) -> float:
        """
        
        """
        if self.__millinewtons != None:
            return self.__millinewtons
        self.__millinewtons = self.__convert_from_base(ForceUnits.Millinewton)
        return self.__millinewtons

    
    @property
    def decanewtons(self) -> float:
        """
        
        """
        if self.__decanewtons != None:
            return self.__decanewtons
        self.__decanewtons = self.__convert_from_base(ForceUnits.Decanewton)
        return self.__decanewtons

    
    @property
    def kilonewtons(self) -> float:
        """
        
        """
        if self.__kilonewtons != None:
            return self.__kilonewtons
        self.__kilonewtons = self.__convert_from_base(ForceUnits.Kilonewton)
        return self.__kilonewtons

    
    @property
    def meganewtons(self) -> float:
        """
        
        """
        if self.__meganewtons != None:
            return self.__meganewtons
        self.__meganewtons = self.__convert_from_base(ForceUnits.Meganewton)
        return self.__meganewtons

    
    @property
    def kilopounds_force(self) -> float:
        """
        
        """
        if self.__kilopounds_force != None:
            return self.__kilopounds_force
        self.__kilopounds_force = self.__convert_from_base(ForceUnits.KilopoundForce)
        return self.__kilopounds_force

    
    def to_string(self, unit: ForceUnits = ForceUnits.Newton) -> string:
        """
        Format the Force to string.
        Note! the default format for Force is Newton.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ForceUnits.Dyn:
            return f"""{self.dyne} dyn"""
        
        if unit == ForceUnits.KilogramForce:
            return f"""{self.kilograms_force} kgf"""
        
        if unit == ForceUnits.TonneForce:
            return f"""{self.tonnes_force} tf"""
        
        if unit == ForceUnits.Newton:
            return f"""{self.newtons} N"""
        
        if unit == ForceUnits.KiloPond:
            return f"""{self.kilo_ponds} kp"""
        
        if unit == ForceUnits.Poundal:
            return f"""{self.poundals} pdl"""
        
        if unit == ForceUnits.PoundForce:
            return f"""{self.pounds_force} lbf"""
        
        if unit == ForceUnits.OunceForce:
            return f"""{self.ounce_force} ozf"""
        
        if unit == ForceUnits.ShortTonForce:
            return f"""{self.short_tons_force} tf (short)"""
        
        if unit == ForceUnits.Micronewton:
            return f"""{self.micronewtons} """
        
        if unit == ForceUnits.Millinewton:
            return f"""{self.millinewtons} """
        
        if unit == ForceUnits.Decanewton:
            return f"""{self.decanewtons} """
        
        if unit == ForceUnits.Kilonewton:
            return f"""{self.kilonewtons} """
        
        if unit == ForceUnits.Meganewton:
            return f"""{self.meganewtons} """
        
        if unit == ForceUnits.KilopoundForce:
            return f"""{self.kilopounds_force} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: ForceUnits = ForceUnits.Newton) -> string:
        """
        Get Force unit abbreviation.
        Note! the default abbreviation for Force is Newton.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ForceUnits.Dyn:
            return """dyn"""
        
        if unit_abbreviation == ForceUnits.KilogramForce:
            return """kgf"""
        
        if unit_abbreviation == ForceUnits.TonneForce:
            return """tf"""
        
        if unit_abbreviation == ForceUnits.Newton:
            return """N"""
        
        if unit_abbreviation == ForceUnits.KiloPond:
            return """kp"""
        
        if unit_abbreviation == ForceUnits.Poundal:
            return """pdl"""
        
        if unit_abbreviation == ForceUnits.PoundForce:
            return """lbf"""
        
        if unit_abbreviation == ForceUnits.OunceForce:
            return """ozf"""
        
        if unit_abbreviation == ForceUnits.ShortTonForce:
            return """tf (short)"""
        
        if unit_abbreviation == ForceUnits.Micronewton:
            return """"""
        
        if unit_abbreviation == ForceUnits.Millinewton:
            return """"""
        
        if unit_abbreviation == ForceUnits.Decanewton:
            return """"""
        
        if unit_abbreviation == ForceUnits.Kilonewton:
            return """"""
        
        if unit_abbreviation == ForceUnits.Meganewton:
            return """"""
        
        if unit_abbreviation == ForceUnits.KilopoundForce:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, Force):
            raise TypeError("unsupported operand type(s) for +: 'Force' and '{}'".format(type(other).__name__))
        return Force(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, Force):
            raise TypeError("unsupported operand type(s) for *: 'Force' and '{}'".format(type(other).__name__))
        return Force(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, Force):
            raise TypeError("unsupported operand type(s) for -: 'Force' and '{}'".format(type(other).__name__))
        return Force(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, Force):
            raise TypeError("unsupported operand type(s) for /: 'Force' and '{}'".format(type(other).__name__))
        return Force(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, Force):
            raise TypeError("unsupported operand type(s) for %: 'Force' and '{}'".format(type(other).__name__))
        return Force(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, Force):
            raise TypeError("unsupported operand type(s) for **: 'Force' and '{}'".format(type(other).__name__))
        return Force(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, Force):
            raise TypeError("unsupported operand type(s) for ==: 'Force' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, Force):
            raise TypeError("unsupported operand type(s) for <: 'Force' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, Force):
            raise TypeError("unsupported operand type(s) for >: 'Force' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, Force):
            raise TypeError("unsupported operand type(s) for <=: 'Force' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, Force):
            raise TypeError("unsupported operand type(s) for >=: 'Force' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class DynamicViscosityUnits(Enum):
        """
            DynamicViscosityUnits enumeration
        """
        
        NewtonSecondPerMeterSquared = 'newton_second_per_meter_squared'
        """
            
        """
        
        PascalSecond = 'pascal_second'
        """
            
        """
        
        Poise = 'poise'
        """
            
        """
        
        Reyn = 'reyn'
        """
            
        """
        
        PoundForceSecondPerSquareInch = 'pound_force_second_per_square_inch'
        """
            
        """
        
        PoundForceSecondPerSquareFoot = 'pound_force_second_per_square_foot'
        """
            
        """
        
        PoundPerFootSecond = 'pound_per_foot_second'
        """
            
        """
        
        MillipascalSecond = 'millipascal_second'
        """
            
        """
        
        MicropascalSecond = 'micropascal_second'
        """
            
        """
        
        Centipoise = 'centipoise'
        """
            
        """
        

class DynamicViscosity:
    """
    The dynamic (shear) viscosity of a fluid expresses its resistance to shearing flows, where adjacent layers move parallel to each other with different speeds

    Args:
        value (float): The value.
        from_unit (DynamicViscosityUnits): The DynamicViscosity unit to create from, The default unit is NewtonSecondPerMeterSquared
    """
    def __init__(self, value: float, from_unit: DynamicViscosityUnits = DynamicViscosityUnits.NewtonSecondPerMeterSquared):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__newton_seconds_per_meter_squared = None
        
        self.__pascal_seconds = None
        
        self.__poise = None
        
        self.__reyns = None
        
        self.__pounds_force_second_per_square_inch = None
        
        self.__pounds_force_second_per_square_foot = None
        
        self.__pounds_per_foot_second = None
        
        self.__millipascal_seconds = None
        
        self.__micropascal_seconds = None
        
        self.__centipoise = None
        

    def __convert_from_base(self, from_unit: DynamicViscosityUnits) -> float:
        value = self.__value
        
        if from_unit == DynamicViscosityUnits.NewtonSecondPerMeterSquared:
            return (value)
        
        if from_unit == DynamicViscosityUnits.PascalSecond:
            return (value)
        
        if from_unit == DynamicViscosityUnits.Poise:
            return (value * 10)
        
        if from_unit == DynamicViscosityUnits.Reyn:
            return (value / 6.8947572931683613e3)
        
        if from_unit == DynamicViscosityUnits.PoundForceSecondPerSquareInch:
            return (value / 6.8947572931683613e3)
        
        if from_unit == DynamicViscosityUnits.PoundForceSecondPerSquareFoot:
            return (value / 4.7880258980335843e1)
        
        if from_unit == DynamicViscosityUnits.PoundPerFootSecond:
            return (value / 1.4881639)
        
        if from_unit == DynamicViscosityUnits.MillipascalSecond:
            return ((value) / 0.001)
        
        if from_unit == DynamicViscosityUnits.MicropascalSecond:
            return ((value) / 1e-06)
        
        if from_unit == DynamicViscosityUnits.Centipoise:
            return ((value * 10) / 0.01)
        
        return None


    def __convert_to_base(self, value: float, to_unit: DynamicViscosityUnits) -> float:
        
        if to_unit == DynamicViscosityUnits.NewtonSecondPerMeterSquared:
            return (value)
        
        if to_unit == DynamicViscosityUnits.PascalSecond:
            return (value)
        
        if to_unit == DynamicViscosityUnits.Poise:
            return (value / 10)
        
        if to_unit == DynamicViscosityUnits.Reyn:
            return (value * 6.8947572931683613e3)
        
        if to_unit == DynamicViscosityUnits.PoundForceSecondPerSquareInch:
            return (value * 6.8947572931683613e3)
        
        if to_unit == DynamicViscosityUnits.PoundForceSecondPerSquareFoot:
            return (value * 4.7880258980335843e1)
        
        if to_unit == DynamicViscosityUnits.PoundPerFootSecond:
            return (value * 1.4881639)
        
        if to_unit == DynamicViscosityUnits.MillipascalSecond:
            return ((value) * 0.001)
        
        if to_unit == DynamicViscosityUnits.MicropascalSecond:
            return ((value) * 1e-06)
        
        if to_unit == DynamicViscosityUnits.Centipoise:
            return ((value / 10) * 0.01)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_newton_seconds_per_meter_squared(newton_seconds_per_meter_squared: float):
        """
        Create a new instance of DynamicViscosity from a value in newton_seconds_per_meter_squared.

        

        :param meters: The DynamicViscosity value in newton_seconds_per_meter_squared.
        :type newton_seconds_per_meter_squared: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(newton_seconds_per_meter_squared, DynamicViscosityUnits.NewtonSecondPerMeterSquared)

    
    @staticmethod
    def from_pascal_seconds(pascal_seconds: float):
        """
        Create a new instance of DynamicViscosity from a value in pascal_seconds.

        

        :param meters: The DynamicViscosity value in pascal_seconds.
        :type pascal_seconds: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(pascal_seconds, DynamicViscosityUnits.PascalSecond)

    
    @staticmethod
    def from_poise(poise: float):
        """
        Create a new instance of DynamicViscosity from a value in poise.

        

        :param meters: The DynamicViscosity value in poise.
        :type poise: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(poise, DynamicViscosityUnits.Poise)

    
    @staticmethod
    def from_reyns(reyns: float):
        """
        Create a new instance of DynamicViscosity from a value in reyns.

        

        :param meters: The DynamicViscosity value in reyns.
        :type reyns: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(reyns, DynamicViscosityUnits.Reyn)

    
    @staticmethod
    def from_pounds_force_second_per_square_inch(pounds_force_second_per_square_inch: float):
        """
        Create a new instance of DynamicViscosity from a value in pounds_force_second_per_square_inch.

        

        :param meters: The DynamicViscosity value in pounds_force_second_per_square_inch.
        :type pounds_force_second_per_square_inch: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(pounds_force_second_per_square_inch, DynamicViscosityUnits.PoundForceSecondPerSquareInch)

    
    @staticmethod
    def from_pounds_force_second_per_square_foot(pounds_force_second_per_square_foot: float):
        """
        Create a new instance of DynamicViscosity from a value in pounds_force_second_per_square_foot.

        

        :param meters: The DynamicViscosity value in pounds_force_second_per_square_foot.
        :type pounds_force_second_per_square_foot: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(pounds_force_second_per_square_foot, DynamicViscosityUnits.PoundForceSecondPerSquareFoot)

    
    @staticmethod
    def from_pounds_per_foot_second(pounds_per_foot_second: float):
        """
        Create a new instance of DynamicViscosity from a value in pounds_per_foot_second.

        

        :param meters: The DynamicViscosity value in pounds_per_foot_second.
        :type pounds_per_foot_second: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(pounds_per_foot_second, DynamicViscosityUnits.PoundPerFootSecond)

    
    @staticmethod
    def from_millipascal_seconds(millipascal_seconds: float):
        """
        Create a new instance of DynamicViscosity from a value in millipascal_seconds.

        

        :param meters: The DynamicViscosity value in millipascal_seconds.
        :type millipascal_seconds: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(millipascal_seconds, DynamicViscosityUnits.MillipascalSecond)

    
    @staticmethod
    def from_micropascal_seconds(micropascal_seconds: float):
        """
        Create a new instance of DynamicViscosity from a value in micropascal_seconds.

        

        :param meters: The DynamicViscosity value in micropascal_seconds.
        :type micropascal_seconds: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(micropascal_seconds, DynamicViscosityUnits.MicropascalSecond)

    
    @staticmethod
    def from_centipoise(centipoise: float):
        """
        Create a new instance of DynamicViscosity from a value in centipoise.

        

        :param meters: The DynamicViscosity value in centipoise.
        :type centipoise: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(centipoise, DynamicViscosityUnits.Centipoise)

    
    @property
    def newton_seconds_per_meter_squared(self) -> float:
        """
        
        """
        if self.__newton_seconds_per_meter_squared != None:
            return self.__newton_seconds_per_meter_squared
        self.__newton_seconds_per_meter_squared = self.__convert_from_base(DynamicViscosityUnits.NewtonSecondPerMeterSquared)
        return self.__newton_seconds_per_meter_squared

    
    @property
    def pascal_seconds(self) -> float:
        """
        
        """
        if self.__pascal_seconds != None:
            return self.__pascal_seconds
        self.__pascal_seconds = self.__convert_from_base(DynamicViscosityUnits.PascalSecond)
        return self.__pascal_seconds

    
    @property
    def poise(self) -> float:
        """
        
        """
        if self.__poise != None:
            return self.__poise
        self.__poise = self.__convert_from_base(DynamicViscosityUnits.Poise)
        return self.__poise

    
    @property
    def reyns(self) -> float:
        """
        
        """
        if self.__reyns != None:
            return self.__reyns
        self.__reyns = self.__convert_from_base(DynamicViscosityUnits.Reyn)
        return self.__reyns

    
    @property
    def pounds_force_second_per_square_inch(self) -> float:
        """
        
        """
        if self.__pounds_force_second_per_square_inch != None:
            return self.__pounds_force_second_per_square_inch
        self.__pounds_force_second_per_square_inch = self.__convert_from_base(DynamicViscosityUnits.PoundForceSecondPerSquareInch)
        return self.__pounds_force_second_per_square_inch

    
    @property
    def pounds_force_second_per_square_foot(self) -> float:
        """
        
        """
        if self.__pounds_force_second_per_square_foot != None:
            return self.__pounds_force_second_per_square_foot
        self.__pounds_force_second_per_square_foot = self.__convert_from_base(DynamicViscosityUnits.PoundForceSecondPerSquareFoot)
        return self.__pounds_force_second_per_square_foot

    
    @property
    def pounds_per_foot_second(self) -> float:
        """
        
        """
        if self.__pounds_per_foot_second != None:
            return self.__pounds_per_foot_second
        self.__pounds_per_foot_second = self.__convert_from_base(DynamicViscosityUnits.PoundPerFootSecond)
        return self.__pounds_per_foot_second

    
    @property
    def millipascal_seconds(self) -> float:
        """
        
        """
        if self.__millipascal_seconds != None:
            return self.__millipascal_seconds
        self.__millipascal_seconds = self.__convert_from_base(DynamicViscosityUnits.MillipascalSecond)
        return self.__millipascal_seconds

    
    @property
    def micropascal_seconds(self) -> float:
        """
        
        """
        if self.__micropascal_seconds != None:
            return self.__micropascal_seconds
        self.__micropascal_seconds = self.__convert_from_base(DynamicViscosityUnits.MicropascalSecond)
        return self.__micropascal_seconds

    
    @property
    def centipoise(self) -> float:
        """
        
        """
        if self.__centipoise != None:
            return self.__centipoise
        self.__centipoise = self.__convert_from_base(DynamicViscosityUnits.Centipoise)
        return self.__centipoise

    
    def to_string(self, unit: DynamicViscosityUnits = DynamicViscosityUnits.NewtonSecondPerMeterSquared) -> string:
        """
        Format the DynamicViscosity to string.
        Note! the default format for DynamicViscosity is NewtonSecondPerMeterSquared.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == DynamicViscosityUnits.NewtonSecondPerMeterSquared:
            return f"""{self.newton_seconds_per_meter_squared} Ns/m²"""
        
        if unit == DynamicViscosityUnits.PascalSecond:
            return f"""{self.pascal_seconds} Pa·s"""
        
        if unit == DynamicViscosityUnits.Poise:
            return f"""{self.poise} P"""
        
        if unit == DynamicViscosityUnits.Reyn:
            return f"""{self.reyns} reyn"""
        
        if unit == DynamicViscosityUnits.PoundForceSecondPerSquareInch:
            return f"""{self.pounds_force_second_per_square_inch} lbf·s/in²"""
        
        if unit == DynamicViscosityUnits.PoundForceSecondPerSquareFoot:
            return f"""{self.pounds_force_second_per_square_foot} lbf·s/ft²"""
        
        if unit == DynamicViscosityUnits.PoundPerFootSecond:
            return f"""{self.pounds_per_foot_second} lb/ft·s"""
        
        if unit == DynamicViscosityUnits.MillipascalSecond:
            return f"""{self.millipascal_seconds} """
        
        if unit == DynamicViscosityUnits.MicropascalSecond:
            return f"""{self.micropascal_seconds} """
        
        if unit == DynamicViscosityUnits.Centipoise:
            return f"""{self.centipoise} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: DynamicViscosityUnits = DynamicViscosityUnits.NewtonSecondPerMeterSquared) -> string:
        """
        Get DynamicViscosity unit abbreviation.
        Note! the default abbreviation for DynamicViscosity is NewtonSecondPerMeterSquared.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == DynamicViscosityUnits.NewtonSecondPerMeterSquared:
            return """Ns/m²"""
        
        if unit_abbreviation == DynamicViscosityUnits.PascalSecond:
            return """Pa·s"""
        
        if unit_abbreviation == DynamicViscosityUnits.Poise:
            return """P"""
        
        if unit_abbreviation == DynamicViscosityUnits.Reyn:
            return """reyn"""
        
        if unit_abbreviation == DynamicViscosityUnits.PoundForceSecondPerSquareInch:
            return """lbf·s/in²"""
        
        if unit_abbreviation == DynamicViscosityUnits.PoundForceSecondPerSquareFoot:
            return """lbf·s/ft²"""
        
        if unit_abbreviation == DynamicViscosityUnits.PoundPerFootSecond:
            return """lb/ft·s"""
        
        if unit_abbreviation == DynamicViscosityUnits.MillipascalSecond:
            return """"""
        
        if unit_abbreviation == DynamicViscosityUnits.MicropascalSecond:
            return """"""
        
        if unit_abbreviation == DynamicViscosityUnits.Centipoise:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, DynamicViscosity):
            raise TypeError("unsupported operand type(s) for +: 'DynamicViscosity' and '{}'".format(type(other).__name__))
        return DynamicViscosity(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, DynamicViscosity):
            raise TypeError("unsupported operand type(s) for *: 'DynamicViscosity' and '{}'".format(type(other).__name__))
        return DynamicViscosity(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, DynamicViscosity):
            raise TypeError("unsupported operand type(s) for -: 'DynamicViscosity' and '{}'".format(type(other).__name__))
        return DynamicViscosity(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, DynamicViscosity):
            raise TypeError("unsupported operand type(s) for /: 'DynamicViscosity' and '{}'".format(type(other).__name__))
        return DynamicViscosity(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, DynamicViscosity):
            raise TypeError("unsupported operand type(s) for %: 'DynamicViscosity' and '{}'".format(type(other).__name__))
        return DynamicViscosity(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, DynamicViscosity):
            raise TypeError("unsupported operand type(s) for **: 'DynamicViscosity' and '{}'".format(type(other).__name__))
        return DynamicViscosity(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, DynamicViscosity):
            raise TypeError("unsupported operand type(s) for ==: 'DynamicViscosity' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, DynamicViscosity):
            raise TypeError("unsupported operand type(s) for <: 'DynamicViscosity' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, DynamicViscosity):
            raise TypeError("unsupported operand type(s) for >: 'DynamicViscosity' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, DynamicViscosity):
            raise TypeError("unsupported operand type(s) for <=: 'DynamicViscosity' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, DynamicViscosity):
            raise TypeError("unsupported operand type(s) for >=: 'DynamicViscosity' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value


from enum import Enum
import math
import string


class HeatTransferCoefficientUnits(Enum):
        """
            HeatTransferCoefficientUnits enumeration
        """
        
        WattPerSquareMeterKelvin = 'watt_per_square_meter_kelvin'
        """
            
        """
        
        WattPerSquareMeterCelsius = 'watt_per_square_meter_celsius'
        """
            
        """
        
        BtuPerHourSquareFootDegreeFahrenheit = 'btu_per_hour_square_foot_degree_fahrenheit'
        """
            
        """
        
        CaloriePerHourSquareMeterDegreeCelsius = 'calorie_per_hour_square_meter_degree_celsius'
        """
            
        """
        
        KilocaloriePerHourSquareMeterDegreeCelsius = 'kilocalorie_per_hour_square_meter_degree_celsius'
        """
            
        """
        

class HeatTransferCoefficient:
    """
    The heat transfer coefficient or film coefficient, or film effectiveness, in thermodynamics and in mechanics is the proportionality constant between the heat flux and the thermodynamic driving force for the flow of heat (i.e., the temperature difference, ΔT)

    Args:
        value (float): The value.
        from_unit (HeatTransferCoefficientUnits): The HeatTransferCoefficient unit to create from, The default unit is WattPerSquareMeterKelvin
    """
    def __init__(self, value: float, from_unit: HeatTransferCoefficientUnits = HeatTransferCoefficientUnits.WattPerSquareMeterKelvin):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self.__value = self.__convert_to_base(value, from_unit)
        
        self.__watts_per_square_meter_kelvin = None
        
        self.__watts_per_square_meter_celsius = None
        
        self.__btus_per_hour_square_foot_degree_fahrenheit = None
        
        self.__calories_per_hour_square_meter_degree_celsius = None
        
        self.__kilocalories_per_hour_square_meter_degree_celsius = None
        

    def __convert_from_base(self, from_unit: HeatTransferCoefficientUnits) -> float:
        value = self.__value
        
        if from_unit == HeatTransferCoefficientUnits.WattPerSquareMeterKelvin:
            return (value)
        
        if from_unit == HeatTransferCoefficientUnits.WattPerSquareMeterCelsius:
            return (value)
        
        if from_unit == HeatTransferCoefficientUnits.BtuPerHourSquareFootDegreeFahrenheit:
            return (value / 5.6782633411134878)
        
        if from_unit == HeatTransferCoefficientUnits.CaloriePerHourSquareMeterDegreeCelsius:
            return ((value / 4.1868) * 3600)
        
        if from_unit == HeatTransferCoefficientUnits.KilocaloriePerHourSquareMeterDegreeCelsius:
            return (((value / 4.1868) * 3600) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: HeatTransferCoefficientUnits) -> float:
        
        if to_unit == HeatTransferCoefficientUnits.WattPerSquareMeterKelvin:
            return (value)
        
        if to_unit == HeatTransferCoefficientUnits.WattPerSquareMeterCelsius:
            return (value)
        
        if to_unit == HeatTransferCoefficientUnits.BtuPerHourSquareFootDegreeFahrenheit:
            return (value * 5.6782633411134878)
        
        if to_unit == HeatTransferCoefficientUnits.CaloriePerHourSquareMeterDegreeCelsius:
            return ((value * 4.1868) / 3600)
        
        if to_unit == HeatTransferCoefficientUnits.KilocaloriePerHourSquareMeterDegreeCelsius:
            return (((value * 4.1868) / 3600) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self.__value

    
    @staticmethod
    def from_watts_per_square_meter_kelvin(watts_per_square_meter_kelvin: float):
        """
        Create a new instance of HeatTransferCoefficient from a value in watts_per_square_meter_kelvin.

        

        :param meters: The HeatTransferCoefficient value in watts_per_square_meter_kelvin.
        :type watts_per_square_meter_kelvin: float
        :return: A new instance of HeatTransferCoefficient.
        :rtype: HeatTransferCoefficient
        """
        return HeatTransferCoefficient(watts_per_square_meter_kelvin, HeatTransferCoefficientUnits.WattPerSquareMeterKelvin)

    
    @staticmethod
    def from_watts_per_square_meter_celsius(watts_per_square_meter_celsius: float):
        """
        Create a new instance of HeatTransferCoefficient from a value in watts_per_square_meter_celsius.

        

        :param meters: The HeatTransferCoefficient value in watts_per_square_meter_celsius.
        :type watts_per_square_meter_celsius: float
        :return: A new instance of HeatTransferCoefficient.
        :rtype: HeatTransferCoefficient
        """
        return HeatTransferCoefficient(watts_per_square_meter_celsius, HeatTransferCoefficientUnits.WattPerSquareMeterCelsius)

    
    @staticmethod
    def from_btus_per_hour_square_foot_degree_fahrenheit(btus_per_hour_square_foot_degree_fahrenheit: float):
        """
        Create a new instance of HeatTransferCoefficient from a value in btus_per_hour_square_foot_degree_fahrenheit.

        

        :param meters: The HeatTransferCoefficient value in btus_per_hour_square_foot_degree_fahrenheit.
        :type btus_per_hour_square_foot_degree_fahrenheit: float
        :return: A new instance of HeatTransferCoefficient.
        :rtype: HeatTransferCoefficient
        """
        return HeatTransferCoefficient(btus_per_hour_square_foot_degree_fahrenheit, HeatTransferCoefficientUnits.BtuPerHourSquareFootDegreeFahrenheit)

    
    @staticmethod
    def from_calories_per_hour_square_meter_degree_celsius(calories_per_hour_square_meter_degree_celsius: float):
        """
        Create a new instance of HeatTransferCoefficient from a value in calories_per_hour_square_meter_degree_celsius.

        

        :param meters: The HeatTransferCoefficient value in calories_per_hour_square_meter_degree_celsius.
        :type calories_per_hour_square_meter_degree_celsius: float
        :return: A new instance of HeatTransferCoefficient.
        :rtype: HeatTransferCoefficient
        """
        return HeatTransferCoefficient(calories_per_hour_square_meter_degree_celsius, HeatTransferCoefficientUnits.CaloriePerHourSquareMeterDegreeCelsius)

    
    @staticmethod
    def from_kilocalories_per_hour_square_meter_degree_celsius(kilocalories_per_hour_square_meter_degree_celsius: float):
        """
        Create a new instance of HeatTransferCoefficient from a value in kilocalories_per_hour_square_meter_degree_celsius.

        

        :param meters: The HeatTransferCoefficient value in kilocalories_per_hour_square_meter_degree_celsius.
        :type kilocalories_per_hour_square_meter_degree_celsius: float
        :return: A new instance of HeatTransferCoefficient.
        :rtype: HeatTransferCoefficient
        """
        return HeatTransferCoefficient(kilocalories_per_hour_square_meter_degree_celsius, HeatTransferCoefficientUnits.KilocaloriePerHourSquareMeterDegreeCelsius)

    
    @property
    def watts_per_square_meter_kelvin(self) -> float:
        """
        
        """
        if self.__watts_per_square_meter_kelvin != None:
            return self.__watts_per_square_meter_kelvin
        self.__watts_per_square_meter_kelvin = self.__convert_from_base(HeatTransferCoefficientUnits.WattPerSquareMeterKelvin)
        return self.__watts_per_square_meter_kelvin

    
    @property
    def watts_per_square_meter_celsius(self) -> float:
        """
        
        """
        if self.__watts_per_square_meter_celsius != None:
            return self.__watts_per_square_meter_celsius
        self.__watts_per_square_meter_celsius = self.__convert_from_base(HeatTransferCoefficientUnits.WattPerSquareMeterCelsius)
        return self.__watts_per_square_meter_celsius

    
    @property
    def btus_per_hour_square_foot_degree_fahrenheit(self) -> float:
        """
        
        """
        if self.__btus_per_hour_square_foot_degree_fahrenheit != None:
            return self.__btus_per_hour_square_foot_degree_fahrenheit
        self.__btus_per_hour_square_foot_degree_fahrenheit = self.__convert_from_base(HeatTransferCoefficientUnits.BtuPerHourSquareFootDegreeFahrenheit)
        return self.__btus_per_hour_square_foot_degree_fahrenheit

    
    @property
    def calories_per_hour_square_meter_degree_celsius(self) -> float:
        """
        
        """
        if self.__calories_per_hour_square_meter_degree_celsius != None:
            return self.__calories_per_hour_square_meter_degree_celsius
        self.__calories_per_hour_square_meter_degree_celsius = self.__convert_from_base(HeatTransferCoefficientUnits.CaloriePerHourSquareMeterDegreeCelsius)
        return self.__calories_per_hour_square_meter_degree_celsius

    
    @property
    def kilocalories_per_hour_square_meter_degree_celsius(self) -> float:
        """
        
        """
        if self.__kilocalories_per_hour_square_meter_degree_celsius != None:
            return self.__kilocalories_per_hour_square_meter_degree_celsius
        self.__kilocalories_per_hour_square_meter_degree_celsius = self.__convert_from_base(HeatTransferCoefficientUnits.KilocaloriePerHourSquareMeterDegreeCelsius)
        return self.__kilocalories_per_hour_square_meter_degree_celsius

    
    def to_string(self, unit: HeatTransferCoefficientUnits = HeatTransferCoefficientUnits.WattPerSquareMeterKelvin) -> string:
        """
        Format the HeatTransferCoefficient to string.
        Note! the default format for HeatTransferCoefficient is WattPerSquareMeterKelvin.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == HeatTransferCoefficientUnits.WattPerSquareMeterKelvin:
            return f"""{self.watts_per_square_meter_kelvin} W/m²·K"""
        
        if unit == HeatTransferCoefficientUnits.WattPerSquareMeterCelsius:
            return f"""{self.watts_per_square_meter_celsius} W/m²·°C"""
        
        if unit == HeatTransferCoefficientUnits.BtuPerHourSquareFootDegreeFahrenheit:
            return f"""{self.btus_per_hour_square_foot_degree_fahrenheit} Btu/h·ft²·°F"""
        
        if unit == HeatTransferCoefficientUnits.CaloriePerHourSquareMeterDegreeCelsius:
            return f"""{self.calories_per_hour_square_meter_degree_celsius} kcal/h·m²·°C"""
        
        if unit == HeatTransferCoefficientUnits.KilocaloriePerHourSquareMeterDegreeCelsius:
            return f"""{self.kilocalories_per_hour_square_meter_degree_celsius} """
        
        return f'{self.__value}'


    def get_unit_abbreviation(self, unit_abbreviation: HeatTransferCoefficientUnits = HeatTransferCoefficientUnits.WattPerSquareMeterKelvin) -> string:
        """
        Get HeatTransferCoefficient unit abbreviation.
        Note! the default abbreviation for HeatTransferCoefficient is WattPerSquareMeterKelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == HeatTransferCoefficientUnits.WattPerSquareMeterKelvin:
            return """W/m²·K"""
        
        if unit_abbreviation == HeatTransferCoefficientUnits.WattPerSquareMeterCelsius:
            return """W/m²·°C"""
        
        if unit_abbreviation == HeatTransferCoefficientUnits.BtuPerHourSquareFootDegreeFahrenheit:
            return """Btu/h·ft²·°F"""
        
        if unit_abbreviation == HeatTransferCoefficientUnits.CaloriePerHourSquareMeterDegreeCelsius:
            return """kcal/h·m²·°C"""
        
        if unit_abbreviation == HeatTransferCoefficientUnits.KilocaloriePerHourSquareMeterDegreeCelsius:
            return """"""
        

    def __str__(self):
        return self.to_string()


    def __add__(self, other):
        if not isinstance(other, HeatTransferCoefficient):
            raise TypeError("unsupported operand type(s) for +: 'HeatTransferCoefficient' and '{}'".format(type(other).__name__))
        return HeatTransferCoefficient(self.__value + other.__value)


    def __mul__(self, other):
        if not isinstance(other, HeatTransferCoefficient):
            raise TypeError("unsupported operand type(s) for *: 'HeatTransferCoefficient' and '{}'".format(type(other).__name__))
        return HeatTransferCoefficient(self.__value * other.__value)


    def __sub__(self, other):
        if not isinstance(other, HeatTransferCoefficient):
            raise TypeError("unsupported operand type(s) for -: 'HeatTransferCoefficient' and '{}'".format(type(other).__name__))
        return HeatTransferCoefficient(self.__value - other.__value)


    def __truediv__(self, other):
        if not isinstance(other, HeatTransferCoefficient):
            raise TypeError("unsupported operand type(s) for /: 'HeatTransferCoefficient' and '{}'".format(type(other).__name__))
        return HeatTransferCoefficient(self.__value / other.__value)


    def __mod__(self, other):
        if not isinstance(other, HeatTransferCoefficient):
            raise TypeError("unsupported operand type(s) for %: 'HeatTransferCoefficient' and '{}'".format(type(other).__name__))
        return HeatTransferCoefficient(self.__value % other.__value)


    def __pow__(self, other):
        if not isinstance(other, HeatTransferCoefficient):
            raise TypeError("unsupported operand type(s) for **: 'HeatTransferCoefficient' and '{}'".format(type(other).__name__))
        return HeatTransferCoefficient(self.__value ** other.__value)


    def __eq__(self, other):
        if not isinstance(other, HeatTransferCoefficient):
            raise TypeError("unsupported operand type(s) for ==: 'HeatTransferCoefficient' and '{}'".format(type(other).__name__))
        return self.__value == other.__value


    def __lt__(self, other):
        if not isinstance(other, HeatTransferCoefficient):
            raise TypeError("unsupported operand type(s) for <: 'HeatTransferCoefficient' and '{}'".format(type(other).__name__))
        return self.__value < other.__value


    def __gt__(self, other):
        if not isinstance(other, HeatTransferCoefficient):
            raise TypeError("unsupported operand type(s) for >: 'HeatTransferCoefficient' and '{}'".format(type(other).__name__))
        return self.__value > other.__value


    def __le__(self, other):
        if not isinstance(other, HeatTransferCoefficient):
            raise TypeError("unsupported operand type(s) for <=: 'HeatTransferCoefficient' and '{}'".format(type(other).__name__))
        return self.__value <= other.__value


    def __ge__(self, other):
        if not isinstance(other, HeatTransferCoefficient):
            raise TypeError("unsupported operand type(s) for >=: 'HeatTransferCoefficient' and '{}'".format(type(other).__name__))
        return self.__value >= other.__value